(* ******************************************************************** *)
(*
 * Project         : HOL-CSP - A Shallow Embedding of CSP in  Isabelle/HOL
 * Version         : 2.0
 *
 * Author          : Burkhart Wolff
 *                   (Based on HOL-CSP 1.0 by Haykal Tej and Burkhart Wolff)
 *
 * This file       : The sequence operator definitions
 *
 * Copyright (c) 2009 Université Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: ROOT.ML 8591 2009-09-26 09:58:14Z brucker $ *)

header{* The Sequence Operator *}

theory  Seq
imports Process 
        (* Legacy *)
begin 

definition  seq :: "['a process,'a process] => 'a process"  (infixl  "`;`" 24)  
where       "P `;` Q \<equiv> Abs_Process
                            ({(t, X). (t, X \<union> {tick}) \<in> F P \<and> tickFree t} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> (t2, X) \<in> F Q} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> t2 \<in> D Q},
                             {t1 @ t2 |t1 t2. t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {t1 @ t2 |t1 t2. t1 @ [tick] \<in> T P \<and> t2 \<in> D Q})"


axioms 
    F_seq   : "F(P `;` Q) =  {(t, X). (t, X \<union> {tick}) \<in> F P \<and> tickFree t} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> (t2, X) \<in> F Q} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> t2 \<in> D Q}"

    D_seq   : "D(P `;` Q) =  {t1 @ t2 |t1 t2. t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {t1 @ t2 |t1 t2. t1 @ [tick] \<in> T P \<and> t2 \<in> D Q}"

    T_seq   : "T(P `;` Q) =  {t. \<exists> X. (t, X \<union> {tick}) \<in> F P \<and> tickFree t} \<union>    (* REALLY ??? *)
                             {t. \<exists> t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> t2 \<in> T Q} \<union>
                             {t1 @ t2 |t1 t2. t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {t1 @ t2 |t1 t2. t1 @ [tick] \<in> T P \<and> t2 \<in> D Q}"

    seq_cont[simp]: "\<lbrakk>cont f; cont g\<rbrakk> \<Longrightarrow> cont (\<lambda> x. f x `;` g x)"

(*
val f = "{(t,X). (t, X Un {tick}) : F P & tickFree t} Un {(t,X). ? t1 t2. t = t1 @ t2 & t1 @ [tick] : T P & (t2, X) : F Q} Un {(t,X). ? t1 t2. t = t1@t2 & t1 : D P & tickFree t1 & front_tickFree t2} Un {(t,X). ? t1 t2. t = t1@t2 & t1@[tick] : T P & t2 : D Q}";

val f1 = "{(t,X). (t, X Un {tick}) : F P & tickFree t} Un {(t,X). ? t1 t2. t = t1 @ t2 & t1 @ [tick] : T P & (t2, X) : F Q}";

val f2 = "{(t,X). ? t1 t2. t = t1@t2 & t1 : D P & tickFree t1 & front_tickFree t2} Un {(t,X). ? t1 t2. t = t1@t2 & t1@[tick] : T P & t2 : D Q}";

val d = "{t .? t1 t2. t = t1@t2 & t1 : D P & tickFree t1 & front_tickFree t2} Un {t .? t1 t2. t = t1@t2 & t1@[tick] : T P & t2 : D Q}";


val t = "{t. ? X. (t, X Un {tick}) : F P & tickFree t} Un {t. ? t1 t2. t = t1 @ t2 & t1 @ [tick] : T P & t2 : T Q} Un" ^ d;
*)

lemma is_processT1_SEQ: "([],{}) : {(t,X). (t, X Un {tick}) : F P & tickFree t} Un {(t,X). ? t1 t2. t = t1 @ t2 & t1 @ [tick] : T P & (t2, X) : F Q} Un {(t,X). ? t1 t2. t = t1@t2 & t1 : D P & tickFree t1 & front_tickFree t2} Un {(t,X). ? t1 t2. t = t1@t2 & t1@[tick] : T P & t2 : D Q}"
  apply (simp)
  apply (subst disj_commute)
  apply (rule disjCI)
  apply (rule disjI1)
  apply (rule conjI)
  apply (rule_tac X= "{}" in F_T)
  apply (simp_all add: is_processT1 is_processT5_S6)
done

lemma is_processT2_SEQ: "! s X. (s, X) :{(t,X). (t, X Un {tick}) : F P & tickFree t} Un {(t,X). ? t1 t2. t = t1 @ t2 & t1 @ [tick] : T P & (t2, X) : F Q} Un {(t,X). ? t1 t2. t = t1@t2 & t1 : D P & tickFree t1 & front_tickFree t2} Un {(t,X). ? t1 t2. t = t1@t2 & t1@[tick] : T P & t2 : D Q} --> front_tickFree s"
  apply (auto)
  apply (rule_tac X="insert tick X" and  P="P" in is_processT2[rule_format])
  apply (simp)
  apply (subst front_tickFree_append)
  apply (rule_tac s="[tick]" in append_T_imp_tickFree)
  apply (simp_all add: is_processT2[rule_format] front_tickFree_append)
  apply (subst front_tickFree_append)
  apply (rule_tac s="[tick]" in append_T_imp_tickFree)
  apply (simp_all add: front_tickFree_append)
  apply (rule_tac P="Q" in is_processT2[rule_format])
  apply (simp add: is_processT8[rule_format])
done

(*  
goal Seq.thy ("! s t. (s @ t, {}) :"^ f1 ^" --> (s, {}) :"^ f );
by(REPEAT(rtac allI 1));
by(simp_tac prod_ss 1);
by(rtac conjI 1);
by(ALLGOALS(rtac impI));
by(REPEAT(etac exE 2));
by(ALLGOALS(etac conjE));
by(cut_inst_tac [("P","(s, {tick}) : F P")] excluded_middle 1);
by(subgoals_tac ["s <= t1 | (? v. v <= t2 & s = t1 @ v)"] 2);
by(REPEAT(SOMEGOAL(etac disjE)));
by(cut_inst_tac [("P","(s, {tick}) : F P")] excluded_middle 3);
by(etac disjE 3);
by(res_inst_tac [("t","t")] append_eq_append 6);
by(atac 6);
by(asm_full_simp_tac (HOL_ss addsimps [tF_append]) 2);
by(REPEAT(SOMEGOAL(etac conjE)));
by(asm_full_simp_tac (list_ss addsimps
                      [append_T_tF RS tF_pref]) 3);
by(ALLGOALS(rtac disjI1));
by(ALLGOALS(rtac disjI1));
by(ALLGOALS(rtac disjI2));
by(etac exE 3);
by(res_inst_tac [("x","t1")] exI 3);
by(res_inst_tac [("x","v")] exI 3);
by(asm_simp_tac HOL_ss 3);
by(res_inst_tac [("t","t2")] is_processT3_S_pref 3);
by(REPEAT(asm_simp_tac HOL_ss 3));
by(ALLGOALS(res_inst_tac [("x","s")] exI));
by(ALLGOALS(res_inst_tac [("x","[]")] exI));
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps [is_processT1,
is_processT3_ST RS (T_F RS(is_processT3_S_pref RS(is_processT5_S4 RS F_T))),
is_processT4_S RS is_processT3_S RS is_processT5_S4 RS F_T])));
val is_processT3_SEQ1 = result();

goal Seq.thy ("! s t. (s @ t, {}) :"^f2^" --> (s, {}) :"^ f );
by(REPEAT(rtac allI 1));
by(simp_tac prod_ss 1);
by(rtac conjI 1);
by(ALLGOALS(rtac impI));
by(REPEAT(ALLGOALS(etac exE)));
by(ALLGOALS(subgoals_tac ["s <= t1 | (? v. v <= t2 & s = t1 @ v)"]));
by(etac disjE 1);
by(etac disjE 4);
by(REPEAT(SOMEGOAL(etac exE) ORELSE SOMEGOAL(etac conjE)));
by(ALLGOALS(asm_simp_tac (HOL_ss addsimps [append_eq_append])));
by(cut_inst_tac [("P","s@[tick] : T P")] excluded_middle 1);
byev[etac disjE 1,REPEAT(rtac disjI1 1),rtac disjI1 2,rtac disjI1 2,
rtac disjI2 2];
by(res_inst_tac [("x","s")] exI 2);
by(res_inst_tac [("x","[]")] exI 2);
byev[rtac disjI1 3,rtac disjI2 3];
by(res_inst_tac [("x","t1")] exI 3);
by(res_inst_tac [("x","v")] exI 3);
by(cut_inst_tac [("P","s@[tick] : T P")] excluded_middle 4);
byev[etac disjE 4,REPEAT(rtac disjI1 4),rtac disjI1 5,rtac disjI1 5,rtac disjI2 5];
by(res_inst_tac [("x","s")] exI 5);
by(res_inst_tac [("x","[]")] exI 5);
byev[rtac disjI1 6,rtac disjI1 6,rtac disjI2 6];
by(res_inst_tac [("x","t1")] exI 6);
by(res_inst_tac [("x","v")] exI 6);
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps
                   [T_F_spec RS sym,is_processT1,is_processT5_S3,
                    is_processT8_S RS is_processT3_S_pref])));
byev[rtac conjI 3,rtac (T_tF RS tF_pref) 4,rtac ftF_pref 2,rtac tF_pref 1];
by(rtac 
((T_F RS is_processT3_S) RS (is_processT3_S_pref RS is_processT5_S3)) 5);
by(ALLGOALS(asm_full_simp_tac (list_ss addsimps [T_F_spec,T_tF])));
val is_processT3_SEQ2 = result();

goal Seq.thy ("! s t. (s @ t, {}) :"^f^" --> (s, {}) :"^ f );
by(REPEAT(rtac allI 1));
by(rtac impI 1);
by(REPEAT(etac UnE 1));
by(res_inst_tac [("x1","t")] (((is_processT3_SEQ1 RS spec) RS spec) RS mp) 1);
by(res_inst_tac [("x1","t")] (((is_processT3_SEQ1 RS spec) RS spec) RS mp) 2);
by(res_inst_tac [("x1","t")] (((is_processT3_SEQ2 RS spec) RS spec) RS mp) 3);
by(res_inst_tac [("x1","t")] (((is_processT3_SEQ2 RS spec) RS spec) RS mp) 4);
by(ALLGOALS(asm_full_simp_tac prod_ss));
val is_processT3_SEQ = result();

goal Seq.thy ("! s X Y. (s, Y) :"^ f ^" & X <= Y --> (s, X) :"^ f );
by(REPEAT(rtac allI 1));
by(rtac impI 1);
by(etac conjE 1);
by(REPEAT(SOMEGOAL(etac UnE)));
by(ALLGOALS(UNFOLD "?X" [Un_assoc]));
by(REPEAT(rtac UnI1 1));
by(rtac UnI1 2);
by(rtac UnI2 2);
by(REPEAT(rtac UnI2 4));
by(rtac UnI2 3);
by(rtac UnI1 3);
by(ALLGOALS(asm_full_simp_tac prod_ss));
by(REPEAT(etac exE 2));
by(res_inst_tac [("x","t1")] exI 2);
by(res_inst_tac [("x","t2")] exI 2);
by(ALLGOALS(asm_simp_tac HOL_ss));
by(res_inst_tac [("Y","Y")] is_processT4_S 2);
by(res_inst_tac [("Y","Y Un {tick}")] is_processT4_S 1);
by(ALLGOALS(asm_simp_tac (set_ss addsimps 
          [Un_mono])));
val is_processT4_SEQ = result();

val prem = goal Seq.thy 
"[|sa = t1 @ t2 ; t1 @ [tick] : T P;\
\ ! c. c : Y --> (! t1 t2. sa @ [c] ~= t1 @ t2 | t1 @ [tick] ~: T P |\
\                                                    (t2, {}) ~: F Q)|]==>\
\ ! c. c : Y --> (t2 @ [c], {}) ~: F Q";
by(cut_facts_tac prem 1);
by(rtac allI 1);
by(rtac impI 1);
by(eres_inst_tac [("x","c")] allE 1);
by(etac impE 1);
by(eres_inst_tac [("x","t1")] allE 2);
by(eres_inst_tac [("x","t2@[c]")] allE 2);
by(ALLGOALS(asm_full_simp_tac list_ss));
val is_processT5_SEQH1 = result();

val prem = goal Seq.thy
"[|(sa, X Un {tick}) : F P; tickFree sa;\
\ ! c. c : Y --> (sa @ [c], {tick}) ~: F P | ~ tickFree (sa @ [c]);\
\ ! c. c : Y -->(! t1 t2. sa @ [c] ~= t1 @ t2 | t1 @ [tick] ~: T P |\
\                                                (t2, {}) ~: F Q)|] ==>\ 
\ ! c. c : Y & c ~= tick --> (sa @ [c], {}) ~: F P ";
by(cut_facts_tac prem 1);
by(etac swap1 1);
by(asm_full_simp_tac (HOL_ss addsimps [Nimp_conj])1);
by(etac exE 1);
by(REPEAT(eres_inst_tac [("x","c")] allE 1));
by(asm_full_simp_tac (HOL_ss addsimps [tF_append,tF_Cons,tF_Nil])1);
by(eres_inst_tac [("x","sa@[c]")] allE 1);
by(eres_inst_tac [("x","[]")] allE 1);
by(etac swap 1);
by(rtac is_processT5_S3 1);
by(ALLGOALS(asm_full_simp_tac (list_ss addsimps
           [is_processT1,T_F_spec,Nil_elem_T])));
val is_processT5_SEQH2 = result();

val prem = goal Seq.thy
"[|(sa, X Un {tick}) : F P; tickFree sa;\
\ ! c. c : Y & c ~= tick --> (sa @ [c], {}) ~: F P |] ==>\ 
\ (sa, X Un Y Un {tick}) : F P & tickFree sa";
by(cut_facts_tac prem 1);
by(res_inst_tac [("t","X Un Y Un {tick}"),("s","X Un {tick} Un (Y-{tick})")] subst 1);
by(rtac set_eqI 1);
by(fast_tac set_cs 1);
by(asm_full_simp_tac (HOL_ss addsimps [Diff_elem_iff,is_processT5_S])1);
val is_processT5_SEQH3 = result();

val prem = goal Seq.thy
"[|(sa, X Un {tick}) : F P; tickFree sa;\
\ ! c. c : Y --> (sa @ [c], {tick}) ~: F P | ~ tickFree (sa @ [c]);\
\ ! c. c : Y -->(! t1 t2. sa @ [c] ~= t1 @ t2 | t1 @ [tick] ~: T P |\
\                                                (t2, {}) ~: F Q)|] ==>\
\ (sa, X Un Y Un {tick}) : F P & tickFree sa";
by(cut_facts_tac prem 1);
by(rtac is_processT5_SEQH3 1);
by(rtac is_processT5_SEQH2 3);
by(ALLGOALS(atac));
val is_processT5_SEQH4 = result();

goal Seq.thy ("! sa X Y.(sa, X) : "^ f1 ^" & (! c. c : Y --> (sa @ [c], {}) ~: "^ f ^") --> (sa, X Un Y) :"^ f1 );
by(REPEAT(rtac allI 1));
by(simp_tac(prod_ss addsimps
      [ex_all_conv,Nconj_eq_disjN,
       Ndisj_eq_conjN,imp_conj,all_conj_distr])1);
by(rtac impI 1);
by(REPEAT((SOMEGOAL(etac conjE) ORELSE SOMEGOAL(etac disjE)) ORELSE SOMEGOAL(etac exE)));
by(subgoals_tac  ["! c. c : Y --> (t2 @ [c], {}) ~: F Q"] 2);
by(res_inst_tac [("sa","sa"),("P","P"),("t1.0","t1")] is_processT5_SEQH1 3);
by(REPEAT(atac 3));
by(rtac disjI2 2);
by(res_inst_tac [("x","t1")] exI 2);
by(res_inst_tac [("x","t2")] exI 2);
by(ALLGOALS(asm_full_simp_tac (set_ss addsimps 
                   [is_processT5_SEQH4,is_processT5_S])));
val is_processT5_SEQ2 = result();

goal Seq.thy ("! sa X Y.(sa, X) : "^ f2 ^" & (! c. c : Y --> (sa @ [c], {}) ~: "^ f ^") --> (sa, X Un Y) :"^ f2 );
by(simp_tac prod_ss 1);
val is_processT5_SEQ1 = result();

goal Seq.thy ("! sa X Y.(sa, X) : "^ f ^" & (! c. c : Y --> (sa @ [c], {}) ~: "^ f ^") --> (sa, X Un Y) :"^ f );
by(REPEAT(rtac allI 1));
by(rtac impI 1);
by(etac conjE 1);
by(REPEAT(etac UnE 1));
by(ALLGOALS(UNFOLD "?X" [Un_assoc]));
byev[rtac UnI1 1,rtac UnI1 2];
byev[rtac UnI2 3,rtac UnI2 4];
by(ALLGOALS(asm_full_simp_tac(HOL_ss addsimps
           [UnI1,is_processT5_SEQ2,
            UnI2,is_processT5_SEQ1])));
val is_processT5_SEQ = result();

goal ProcessType.thy "s@[tick,tick] ~: T P";
by(cut_inst_tac [("P","s@[tick,tick] : T P")] excluded_middle 1);
by(etac disjE 1);
by(subgoals_tac ["front_tickFree(s@[tick]@[tick])"] 2);
by(rtac ((is_processT2_TR RS spec) RS mp) 3);
by(subgoals_tac ["front_tickFree [tick,tick]"] 2);
by(res_inst_tac [("t","s")] ftF_append3 3);
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps [front_tickFree_def,tickFree_def])));
val tick_tick_Nin_T = result();

val prem = goal ProcessType.thy
 "[|s @ [a] = t1 @ t2; t2 ~= []|] ==> t2 = rev(tl(rev t2)) @ [a]";
by(cut_facts_tac prem 1);
by(subgoals_tac ["s = t1 @ rev(tl(rev t2))"] 1);
by(hyp_subst_tac 1);
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps [append_elem_eq_append])));
bind_thm("append_elem_eq_append2",result() RS sym);

goal Seq.thy ("! s X. (s@[tick],{}) :"^ f ^" --> (s,X-{tick}) :"^ f);
by(strip_tac 1);
by(asm_full_simp_tac(list_ss addsimps [disj_assoc,tF_append,NtF_tick])1);
by(REPEAT((SOMEGOAL(etac disjE) ORELSE SOMEGOAL(etac exE)) ORELSE SOMEGOAL(etac conjE)));
byev[rtac disjI2 1,rtac disjI1 1,rtac disjI2 2,rtac disjI2 2,rtac disjI1 2,
REPEAT(rtac disjI2 3)];
by(cut_inst_tac [("P","t2=[]")] excluded_middle 2);
by(etac disjE 2);
by(res_inst_tac [("x","s")] exI 3);
by(res_inst_tac [("x","[]")] exI 3);
by(hyp_subst_tac 3);
by(asm_full_simp_tac list_ss 3);
by(hyp_subst_tac 3);
by(asm_full_simp_tac(list_ss addsimps [tF_append]) 3);
by(rtac is_processT9_S 3);
by(atac 3);
by(ALLGOALS(res_inst_tac [("x","t1")] exI));
by(ALLGOALS(res_inst_tac [("x","rev(tl(rev t2))")] exI));
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps
       [T_tF,tF_append_eq_append_tick2,tF_append_eq_append_tick3])));
by(ALLGOALS(rotate_tac 2));
by(ALLGOALS(etac swap1));
by(ALLGOALS(FOLD "t2" [append_elem_eq_append2]));
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps
       [T_tF RS tF_append_eq_append_tick1])));
by(ALLGOALS(etac swap));
by(rtac is_processT9_S 2);
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps
       [is_processT6])));
val is_processT6_SEQ = result();

goal Seq.thy ("! s t. s :"^ d ^"& tickFree s & front_tickFree t --> s @ t :"^ d);
by(REPEAT(rtac allI 1));
by(rtac impI 1);
by(asm_full_simp_tac prod_ss 1);
by(etac conjE 1);
by(etac disjE 1);
by(REPEAT(ALLGOALS(etac exE)));
by(REPEAT(etac conjE 2));
by(rtac disjI1 1);
by(rtac disjI2 2);
by(ALLGOALS(res_inst_tac [("x","t1")] exI));
by(ALLGOALS(res_inst_tac [("x","t2@t")] exI));
by(REPEAT(SOMEGOAL(etac conjE)));
by(ALLGOALS(hyp_subst_tac));
by(ALLGOALS(asm_full_simp_tac (list1_ss addsimps 
                   [is_processT7_S])));
val is_processT7_SEQ = result();

goal Seq.thy ("! s X. s :"^ d ^"--> (s, X) :"^ f);
by(REPEAT(rtac allI 1));
by(rtac impI 1);
by(asm_full_simp_tac prod_ss 1);
by(etac disjE 1);
by(ALLGOALS(asm_simp_tac HOL_ss));
val is_processT8_SEQ = result();

goal Seq.thy ("! s. s@[tick] :"^ d ^"--> s :"^ d);
by(strip_tac 1);
by(asm_full_simp_tac prod_ss 1);
by(etac disjE 1);
by(REPEAT(ALLGOALS(etac exE)));
by(rtac disjI1 1);
by(rtac disjI2 2);
by(ALLGOALS(cut_inst_tac [("P","t2=[]")] excluded_middle));
by(ALLGOALS(etac disjE));
by(res_inst_tac [("x","s")] exI 2);
by(res_inst_tac [("x","s")] exI 4);
by(res_inst_tac [("x","[]")] exI 2);
by(res_inst_tac [("x","[]")] exI 4);
by(hyp_subst_tac 2);
by(hyp_subst_tac 4);
by(ALLGOALS(asm_full_simp_tac list_ss));
by(REPEAT(SOMEGOAL(etac conjE)));
by(hyp_subst_tac 2);
by(hyp_subst_tac 4);
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps [tF_append,tick_tick_Nin_T])));
by(rtac is_processT9_S 2);
by(atac 2);
by(ALLGOALS(res_inst_tac [("x","t1")] exI));
by(ALLGOALS(res_inst_tac [("x","rev(tl(rev t2))")] exI));
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps
       [T_tF,tF_append_eq_append_tick2,tF_append_eq_append_tick3])));
by(rotate_tac 3 1);
by(etac swap1 1);
by(FOLD "t2" [append_elem_eq_append2] 1);
by(ALLGOALS(asm_full_simp_tac(list_ss addsimps
       [T_tF RS tF_append_eq_append_tick1])));
by(rotate_tac 1 1);
by(etac swap 1);
by(rtac is_processT9_S 1);
by(asm_full_simp_tac list_ss 1);
val is_processT9_SEQ = result();


goalw Seq.thy [is_process_def,DIVERGENCES_def,FAILURES_def] 
("is_process ("^f^","^d^")");
by(simp_tac (HOL_ss addsimps
      [fst_conv,snd_conv])1);
by(REPEAT(SOMEGOAL(rtac conjI)));
by(rtac is_processT1_SEQ 1);
by(rtac is_processT2_SEQ 1);
by(rtac is_processT3_SEQ 1);
by(rtac is_processT4_SEQ 1);
by(rtac is_processT5_SEQ 1);
by(rtac is_processT6_SEQ 1);
by(rtac is_processT7_SEQ 1);
by(rtac is_processT8_SEQ 1);
by(rtac is_processT9_SEQ 1);
val is_process_REP_SEQ = result();

goal Seq.thy ("Rep_Process (Abs_Process ("^f^","^d^")) = ("^f^","^d^")");
by(simp_tac(HOL_ss addsimps
      [CollectI,Process_def,
       Abs_Process_inverse,
       is_process_REP_SEQ])1);
val Rep_Abs_SEQ = result();

goal Seq.thy 
"Rep_Process\
\    (Abs_Process\
\      ({p. (fst p, (snd p) Un {tick}) : F P & tickFree (fst p)} Un\
\      {p. ? t1 t2. fst p = t1 @ t2 & t1 @ [tick] : T P & (t2, snd p) : F Q} Un\
\       {p.\
\        ? t1 t2. fst p = t1 @ t2 & t1 : D P & tickFree t1 & front_tickFree t2}\
\ Un\
 \      {p. ? t1 t2. fst p = t1 @ t2 & t1 @ [tick] : T P & t2 : D Q},\
\       {t1 @ t2 |t1 t2. t1 : D P & tickFree t1 & front_tickFree t2} Un\
\       {t1 @ t2 |t1 t2. t1 @ [tick] : T P & t2 : D Q})) =\
\   ({p. (fst p, (snd p) Un {tick}) : F P & tickFree (fst p)} Un\
\     {p. ? t1 t2. fst p = t1 @ t2 & t1 @ [tick] : T P & (t2, snd p) : F Q} Un\
\       {p.\
 \       ? t1 t2. fst p = t1 @ t2 & t1 : D P & tickFree t1 & front_tickFree t2}\
\ Un\
\       {p. ? t1 t2. fst p = t1 @ t2 & t1 @ [tick] : T P & t2 : D Q},\
\       {t1 @ t2 |t1 t2. t1 : D P & tickFree t1 & front_tickFree t2} Un\
\       {t1 @ t2 |t1 t2. t1 @ [tick] : T P & t2 : D Q})";

by(res_inst_tac
  [("t","%p.(fst p, (snd p) Un {tick}) : F P & tickFree (fst p)")] subst 1);
by(res_inst_tac
  [("t","%p.? t1 t2. fst p = t1 @ t2 & t1 @ [tick] : T P & (t2, snd p) : F Q")] subst 2);
by(res_inst_tac
  [("t","%p.? t1 t2. fst p = t1 @ t2 & t1 : D P & tickFree t1 & front_tickFree t2")] subst 3);
by(res_inst_tac
  [("t","%p.? t1 t2. fst p = t1 @ t2 & t1 @ [tick] : T P & t2 : D Q")] subst 4);
by(REPEAT(rtac split_eq_fst_snd 1));
by(rtac Rep_Abs_SEQ 1);
qed "Rep_Abs_SEQ1";

goalw Seq.thy [SEQ_def] ("F(P `;` Q) ="^f);
by(UNFOLD "F(Abs_Process ?X)" [F_def RS meta_eq_to_obj_eq] 1);
by(simp_tac(list_ss addsimps
      [Rep_Abs_SEQ,FAILURES_def])1);
val F_SEQ = result();

goalw Seq.thy [SEQ_def] ("D(P `;` Q) ="^d);
by(UNFOLD "D(Abs_Process ?X)" [D_def RS meta_eq_to_obj_eq] 1);
by(simp_tac(list_ss addsimps
      [Rep_Abs_SEQ,DIVERGENCES_def])1);
val D_SEQ = result();

goalw Seq.thy [SEQ_def] ("T(P `;` Q) ="^t);
by(UNFOLD "T(Abs_Process ?X)" [T_def RS meta_eq_to_obj_eq] 1);
by(simp_tac(list_ss addsimps
      [Rep_Abs_SEQ1,TRACES_def,
       FAILURES_def,split_def])1);
by(rtac equalityI 1);
by(ALLGOALS(rtac subsetI));
by(ALLGOALS(asm_full_simp_tac list_ss));
by(REPEAT((SOMEGOAL(etac exE) ORELSE SOMEGOAL(etac disjE)) ORELSE SOMEGOAL(etac conjE)));
by(ALLGOALS(asm_simp_tac HOL_ss));
byev[rtac disjI1 2,rtac disjI1 2,rtac disjI2 2,rtac disjI1 3,rtac disjI2 3,rtac disjI2 4];
by(REPEAT(rtac disjI1 1));
by(res_inst_tac [("x","snd a")] exI 1);
by(asm_simp_tac HOL_ss 1);
by(REPEAT(EVERY[
res_inst_tac [("x","t1")] exI 1,
res_inst_tac [("x","t2")] exI 1,
asm_full_simp_tac(HOL_ss addsimps [F_T])1]));
by(res_inst_tac [("x","(x, X)")] exI 1);
by(REPEAT(SOMEGOAL(res_inst_tac [("x","(x, {})")] exI)));
by(ALLGOALS(asm_simp_tac prod_ss));
byev[rtac disjI1 1,rtac disjI1 1,rtac disjI2 1,rtac disjI1 2,rtac disjI2 2,rtac disjI2 3];
by(REPEAT(EVERY[
res_inst_tac [("x","t1")] exI 1,
res_inst_tac [("x","t2")] exI 1,
asm_full_simp_tac(HOL_ss addsimps [T_F])1]));
val T_SEQ = result();
*)

lemma F_D_SEQ_spec: "F (P `;` Q) =
    {(t, X). (t, X \<union> {tick}) \<in> F P \<and> tickFree t} \<union>
    {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> (t2, X) \<in> F Q} \<union>
    {(t, x). t \<in> D (P `;` Q)} "
  by (auto simp: F_seq Un_def D_seq)

lemma F_SEQ_spec: "F (P `;` Q) =
    {(t, X). (t, X \<union> {tick}) \<in> F P \<and> tickFree t} \<union>
    {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> (t2, X) \<in> F Q} \<union>
    {(t, x). \<exists>t1 t2. t = t1 @ t2 \<and> t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2}"
  apply (simp_all add: Un_def F_seq)
  apply (rule equalityI)
  apply (rule subsetI)
  apply (simp)
  apply (erule disjE)
  apply (rule disjI1)
  defer
  apply (erule disjE)
  apply (rule disjI2)
  apply (rule disjI1)
  defer
  apply (erule disjE)
  apply (rule disjI2)
  apply (rule disjI2)
  defer
  apply (rule disjI2)
  apply (rule disjI1)
  defer
  apply (rule subsetI)
  apply (simp)
  apply (erule disjE)
  apply (rule disjI1)
  defer
  apply (erule disjE)
  apply (rule disjI2)
  apply (rule disjI1)
  defer
  apply (rule disjI2)
  apply (rule disjI2)
  apply (rule disjI1)
  apply (auto)
  apply (rule_tac x="t1" in exI, simp)
  apply (simp add: is_processT8_S)
done

(* D P <= {t .? t1 t2. t = t1@t2 & t1 : D P & 
                                  tickFree t1 & front_tickFree t2}
  does hold only if we have the following property as invariant :

                  t@[tick] : D P --> t : D P *)



end

