(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * squareroot_test.thy --- testing integer squareroots
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2003-2007 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2012 Univ. Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: squareroot_test.thy 9446 2012-10-13 16:33:07Z wolff $ *)

(* ML {* add_path "$ISABELLE_HOME/src/HOL/IMP"; *} *)

header{* Program-based Testing: The Squareroot-Example.*}

theory 
  squareroot_test
imports
  "../../src/program_based_testing"
begin

subsection{* The Definition of the Integer-Squareroot Program*}

definition squareroot :: "[loc,loc,loc,loc] \<Rightarrow> com"
where     "squareroot tm sqsum i a ==
                                ((  tm   :== (\<lambda>s. 1));
                                 (( sqsum  :== (\<lambda>s. 1));
                                  ((i    :== (\<lambda>s. 0));
                                    WHILE (\<lambda>s. (s sqsum) <= (s a)) DO
                                       (( i    :== (\<lambda>s. (s i) + 1));
                                        ((tm   :== (\<lambda>s. (s tm) + 2));
                                         (sqsum  :== (\<lambda>s. (s tm) + (s sqsum)))))))
                                 )"


definition  pre   :: assn where   "pre  \<equiv>  \<lambda> x. True"
definition  post  :: "[loc,loc] \<Rightarrow> assn" 
where      "post a i \<equiv>   \<lambda> s. (s i)*(s i)\<le>(s a) \<and> s a < (s i + 1)*(s i + 1)"

definition  inv   :: "[loc,loc,loc,loc] \<Rightarrow> assn"
where      "inv i sqsum tm a \<equiv> \<lambda>s.(s i + 1) * (s i + 1) = s sqsum 
                                   \<and> s tm = (2 * (s i) + 1) 
                                   \<and> (s i) * (s i) <= (s a)"


subsection{* Computing Program Paths and their Path-Constraints*}


lemma derive_pathconds: 
  assumes no_alias : "sqsum \<noteq> i \<and> i \<noteq> sqsum \<and> tm \<noteq> sqsum  \<and> 
                      sqsum \<noteq> tm \<and> sqsum \<noteq> a \<and> a \<noteq> sqsum \<and>
                      tm \<noteq> i \<and> i \<noteq> tm \<and> tm \<noteq> a \<and> a \<noteq> tm  \<and> 
                      a \<noteq> i \<and> i \<noteq> a"
  shows  "\<langle>unfold(3, squareroot tm sqsum i a), s\<rangle> \<longrightarrow>\<^sub>c s'"

  apply(simp add: squareroot_def uf_while)
  apply(rule If_split,simp_all add: update_def no_alias)+
txt{* The resulting proof state capturing the test hypothesis
  as well as the resulting 4 evaluation paths (no entry into
  loop, 1 pass, 2 passes and 3 passes through the loop) looks as 
  follows:
  @{subgoals[display]}
  *}
  oops

(* goal (lemma derive_test_cases, 4 subgoals): 
 1. Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))))) \<le> s a \<Longrightarrow>
    \<langle>WHILE \<lambda>s. s sum
               \<le> s a DO i :== \<lambda>s. Suc (s
  i) ; (tm :== \<lambda>s. Suc (Suc (s tm)) ; sum :== \<lambda>s. s tm + s sum ),s
    (i := Suc (Suc (Suc 0)),
     tm := Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))),
     sum :=
       Suc (Suc (Suc (Suc (Suc (Suc (Suc
(Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0))))))))))))))))\<rangle>
    \<longrightarrow>\<^sub>c s'
 2. \<lbrakk>Suc (Suc (Suc (Suc 0))) \<le> s a;
     \<not> Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))))) \<le> s a\<rbrakk>
    \<Longrightarrow> s' = s
       (i := Suc (Suc 0), tm := Suc (Suc (Suc (Suc (Suc 0)))),
        sum := Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))))))
 3. \<lbrakk>Suc 0 \<le> s a; \<not> Suc (Suc (Suc (Suc 0))) \<le> s a\<rbrakk>
    \<Longrightarrow> s' = s
       (i := Suc 0, tm := Suc (Suc (Suc 0)), sum := Suc (Suc (Suc (Suc 0))))
 4. \<not> Suc 0 \<le> s a \<Longrightarrow> s' = s(tm := Suc 0, sum := Suc 0, i := 0)
 *)


text{* \textbf{Summary}: With this approach, one can synthesize paths 
       and their conditions. *}


subsection{* Testing Specifications *}
thm symbolic_evaluation2

text{* Slow Motion Interactive Version (for demonstrations). *}

lemma whitebox_test: 
  assumes no_alias[simp] : "sqsum \<noteq> i \<and> i \<noteq> sqsum \<and> tm \<noteq> sqsum  \<and> 
                            sqsum \<noteq> tm \<and> sqsum \<noteq> a \<and> a \<noteq> sqsum \<and>
                            tm \<noteq> i \<and> i \<noteq> tm \<and> tm \<noteq> a \<and> a \<noteq> tm  \<and> 
                            a \<noteq> i \<and> i \<noteq> a"
  shows  "|- {pre} squareroot tm sqsum i a {post a i}"

  apply(simp add: squareroot_def pre_def)
  apply(rule_tac n1 = "3" in iffD2[OF symbolic_eval_test])
 
  apply(safe, simp add: Append.simps unfold.simps uf_while)
  apply(simp only: symbolic_evaluation2 
                   memory_model no_alias update_def, 
        safe)
  apply(tactic "ALLGOALS(TestGen.COND' contains_eval 
                                       (TestGen.thyp_ify @{context})
                                       (K all_tac))")

  apply(simp_all) (* global cleanup *)
 
sorry

text{* Automated Version: *}


lemma whitebox_test2: 
  assumes no_alias[simp] : "sqsum \<noteq> i \<and> i \<noteq> sqsum \<and> tm \<noteq> sqsum  \<and> 
                            sqsum \<noteq> tm \<and> sqsum \<noteq> a \<and> a \<noteq> sqsum \<and>
                            tm \<noteq> i \<and> i \<noteq> tm \<and> tm \<noteq> a \<and> a \<noteq> tm  \<and> 
                            a \<noteq> i \<and> i \<noteq> a"
  shows  "|- {pre} squareroot tm sqsum i a {post a i}"

  apply(simp add: squareroot_def pre_def)
  apply(tactic "generate_program_splitter @{context} (@{thms no_alias}) 3 1")

  apply(simp_all) (* global cleanup *)

  txt{* The resulting proof state captures the essence of this
        white box test:

        @{subgoals[display]}
  
        Now testing all paths for compliance to post condition: *}

  apply(simp_all add: no_alias post_def)

  txt{* In this special case---arithmetic constraints---the system can
        even \textbf{verify} these constraints, i.e. the simplifier shows
        that all postconditions follow from the initial constraints
        and the computed relation between pre-state and post state.

         @{subgoals[display]}

        To say it loud and clearly: The white box test decomposes 
        the original specification into a
        test hypothesis for cases with $3^3 = 9 <= s a$ and all other
        cases (e.g. $2^2 = 4 <= s a \land s a < 9$). The latter have been 
        proven automatically. *}
  
(*
 1. \<And>s t. THYP
           (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))))) \<le> s a \<longrightarrow>
            \<langle>WHILE \<lambda>s. s sum
                       \<le> s a DO i :== \<lambda>s.
   Suc (s i) ; (tm :== \<lambda>s. Suc (Suc (s tm)) ; sum :== \<lambda>s. s tm + s sum ),s
            (i := Suc (Suc (Suc 0)),
             tm := Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))),
             sum :=
               Suc (Suc (Suc (Suc (Suc (Suc
   (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0))))))))))))))))\<rangle>
            \<longrightarrow>\<^sub>c t \<longrightarrow> t i * t i \<le> t a \<and> t a < 
                            Suc (t i + (t i + t i * t i))) *)
  oops

 
subsection{* An Alternative Approach with an On-The-Fly generated Explicit Test-Hyp.*}

text{* Recall the rules for the computation of weakest preconditions:
\begin{verbatim}
Hoare.wp_def: wp ?c ?Q == %s. ALL t. (s, t) : C ?c --> ?Q t

Hoare.wp_If:   wp (IF ?b THEN ?c ELSE ?d) ?Q = (%s. (?b s --> wp ?c ?Q s) &
                                              (~ ?b s --> wp ?d ?Q s))
Hoare.wp_Semi: wp (?c; ?d) ?Q = wp ?c (wp ?d ?Q)
Hoare.wp_Ass:  wp (?x :== ?a ) ?Q = (%s. ?Q (s[?x ::= ?a s]))
Hoare.wp_SKIP: wp SKIP ?Q = ?Q
\end{verbatim}                                                                                         
*}

lemma path_exploration_test: 
  assumes no_alias : "sqsum \<noteq> i \<and> i \<noteq> sqsum \<and> tm \<noteq> sqsum  \<and> 
                      sqsum \<noteq> tm \<and> sqsum \<noteq> a \<and> a \<noteq> sqsum \<and>
                      tm \<noteq> i \<and> i \<noteq> tm \<and> tm \<noteq> a \<and> a \<noteq> tm  \<and> 
                      a \<noteq> i \<and> i \<noteq> a"
  shows  "|- {pre} squareroot tm sqsum i a {post a i}"
  txt{* We fire the basic white-box scenario: *}
  apply (rule wp_test [of _ "3"])
  txt{* Given the concrete unfolding factor and the concrete program
        term, standard normalization yields an "Path Exhaustion Theorem"
        with the explicit test hypothesis: *}
  apply(auto simp: squareroot_def update_def no_alias uf_while)
  apply(tactic "ALLGOALS(TestGen.COND' contains_eval 
                                       (TestGen.thyp_ify @{context})
                                       (K all_tac))")
  txt {* and we reach the following instantiation of a white-box 
         test-theorem (with explicit test-hypothesis for the uncovered paths): 
         @{subgoals [display, goals_limit=50]}
         Now we allso perform the "tests" by symbolic execution: *}
  apply(auto simp: no_alias pre_def post_def)
  txt {* which leaves us just with test-hypothesis case; for all
         paths \emph{not} leading to a remaining while, the program
         is correct.
         @{subgoals [display, goals_limit=50]}
      *}

  oops


end
 
