(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: rules-DatatypeNetwork.thy 9426 2012-10-11 08:48:44Z lukasbru $ *)

theory 
  "rules-DatatypeNetwork"
imports
  FWTesting
begin

declare [[testgen_profiling]]

datatype Adr = Workstation | Internet
instance Adr :: adr .. 
type_synonym port = int
type_synonym Networks = "Adr\<times>port"


definition
  workstation::"Networks net" where
  "workstation = {{(a,b). a= Workstation}}"

definition  
   internet :: "Networks net" where
  "internet = {{(a,b). a= Internet}}"

defs (overloaded)
 src_port_def:  "src_port (x::(Networks,'b) packet) \<equiv> (snd o fst o snd) x"
 dest_port_def: "dest_port (x::(Networks,'b) packet) \<equiv> (snd o fst o snd o snd) x"
 subnet_of_def: "subnet_of (x::Networks) \<equiv> {{(a,b). a = fst x}}" 


definition
 not_in_same_net :: "(Networks,DummyContent) packet \<Rightarrow> bool" where
 "not_in_same_net x  =  ((src x \<sqsubset> workstation \<longrightarrow> ( (dest x \<sqsubset> internet))) \<and> 
                        (src x \<sqsubset> internet \<longrightarrow>  dest x \<sqsubset> workstation))"

                     
definition
   policy_00 :: "(Networks, DummyContent) FWPolicy" where
  "policy_00 = deny_all"

definition
   policy_01 :: "(Networks, DummyContent) FWPolicy" where
  "policy_01 = policy_00 ++ allow_from_port_to (80::port) workstation internet"

definition
   policy_02 :: "(Networks, DummyContent) FWPolicy" where
  "policy_02 = policy_01 ++ allow_from_port_to (21::port) workstation internet"

definition
   policy_03 :: "(Networks, DummyContent) FWPolicy" where
  "policy_03 = policy_02 ++ allow_from_port_to (56::port) workstation internet"

definition
   policy_04 :: "(Networks, DummyContent) FWPolicy" where
  "policy_04 = policy_03 ++ allow_from_port_to (34::port) workstation internet "

definition
   policy_05 :: "(Networks, DummyContent) FWPolicy" where
  "policy_05 = policy_04 ++ allow_from_port_to (13::port) workstation internet"

definition
   policy_06 :: "(Networks, DummyContent) FWPolicy" where
  "policy_06 = policy_05 ++ allow_from_port_to (456::port) workstation internet"

definition
   policy_07 :: "(Networks, DummyContent) FWPolicy" where
  "policy_07 = policy_06 ++ allow_from_port_to (443::port) workstation internet"

definition
   policy_08 :: "(Networks, DummyContent) FWPolicy" where
  "policy_08 = policy_07 ++ allow_from_port_to (21::port) workstation internet"

definition
   policy_09 :: "(Networks, DummyContent) FWPolicy" where
  "policy_09 = policy_08 ++ allow_from_port_to (443::port) workstation internet"

definition
   policy_10 :: "(Networks, DummyContent) FWPolicy" where
  "policy_10 = policy_09 ++ allow_from_port_to (342::port) workstation internet"

lemmas UnfoldPolicy =  src_def dest_def in_subnet_def src_port_def dest_port_def
                       workstation_def internet_def policy_00_def 
                       policy_01_def policy_02_def policy_03_def policy_04_def policy_05_def
                       policy_06_def policy_07_def policy_08_def policy_09_def policy_10_def

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_00 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm "Dpolicy_00"

gen_test_data "Dpolicy_00"
thm Dpolicy_00.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_01 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm "Dpolicy_01"

gen_test_data"Dpolicy_01"
thm Dpolicy_01.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_02 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_02"

gen_test_data"Dpolicy_02"
thm Dpolicy_02.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_03 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_03"

gen_test_data"Dpolicy_03"
thm Dpolicy_03.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_04 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_04"

gen_test_data"Dpolicy_04"
thm Dpolicy_04.test_data


test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_05 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_05"

gen_test_data"Dpolicy_05"
thm Dpolicy_05.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_06 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_06"

gen_test_data"Dpolicy_06"
thm Dpolicy_06.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_07 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_07"

gen_test_data"Dpolicy_07"
thm Dpolicy_07.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_08 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_08"

gen_test_data"Dpolicy_08"
thm Dpolicy_08.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_09 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_09"

gen_test_data"Dpolicy_09"
thm Dpolicy_09.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_10 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"Dpolicy_10"

gen_test_data"Dpolicy_10"
thm Dpolicy_10.test_data


write_clocks "time_stats_rulesDatatypeNetwork.tex"

declare [[testgen_profiling = false]]


end
