(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: rules-IntegerNetwork.thy 9426 2012-10-11 08:48:44Z lukasbru $ *)

theory 
  "rules-IntegerNetwork"
imports
  FWTesting
begin

declare [[testgen_profiling]]

definition
  workstation::"adr\<^isub>i\<^isub>p net" where
  "workstation = {{(a,b) . (a=3) }}"
definition
  internet :: "adr\<^isub>i\<^isub>p net" where
  "internet = {{(a,b).  \<not> (a =3)}}"

definition
  not_in_same_net :: "(adr\<^isub>i\<^isub>p,DummyContent) packet \<Rightarrow> bool" where
  "not_in_same_net x  =  ((src x \<sqsubset> workstation \<longrightarrow> ( (dest x \<sqsubset> internet))) \<and> 
  (src x \<sqsubset> internet \<longrightarrow>  dest x \<sqsubset> workstation))"

definition
   policy_00 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_00 = deny_all"
definition
   policy_01 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_01 = policy_00 ++ allow_from_port_to (80::port) workstation internet"
definition
   policy_02 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_02 = policy_01 ++ allow_from_port_to (21::port) workstation internet"
definition
   policy_03 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_03 = policy_02 ++ allow_from_port_to (56::port) workstation internet"
definition
   policy_04 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_04 = policy_03 ++ allow_from_port_to (34::port) workstation internet "
definition
   policy_05 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_05 = policy_04 ++ allow_from_port_to (13::port) workstation internet"
definition
   policy_06 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_06 = policy_05 ++ allow_from_port_to (456::port) workstation internet"
definition
   policy_07 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_07 = policy_06 ++ allow_from_port_to (443::port) workstation internet"
definition
   policy_08 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_08 = policy_07 ++ allow_from_port_to (21::port) workstation internet"
definition
   policy_09 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_09 = policy_08 ++ allow_from_port_to (443::port) workstation internet"
definition
   policy_10 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_10 = policy_09 ++ allow_from_port_to (3142::port) workstation internet"
definition
   policy_11 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_11 = policy_10 ++ allow_from_port_to (1342::port) workstation internet"
definition
   policy_12 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_12 = policy_11 ++ allow_from_port_to (3412::port) workstation internet"
definition
   policy_13 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_13 = policy_12 ++ allow_from_port_to (3421::port) workstation internet"
definition
   policy_14 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_14 = policy_13 ++ allow_from_port_to (3452::port) workstation internet"
definition
   policy_15 :: "(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy" where
  "policy_15 = policy_14 ++ allow_from_port_to (3342::port) workstation internet"

lemmas UnfoldPolicy =  src_def dest_def in_subnet_def adr\<^isub>i\<^isub>pLemmas
                       workstation_def internet_def policy_00_def 
                       policy_01_def policy_02_def policy_03_def policy_04_def policy_05_def
                       policy_06_def policy_07_def policy_08_def policy_09_def policy_10_def
                       policy_11_def policy_12_def policy_13_def policy_14_def policy_15_def


declare [[testgen_profiling]]

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_00 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm "policy_00"

gen_test_data "policy_00"
thm policy_00.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_01 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases "FUT")
apply simp_all
store_test_thm "policy_01"

gen_test_data"policy_01"
thm policy_01.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_02 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_02"

gen_test_data"policy_02"
thm policy_02.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_03 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_03"

gen_test_data"policy_03"
thm policy_03.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_04 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_04"

gen_test_data"policy_04"
thm policy_04.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_05 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_05"

gen_test_data"policy_05"
thm policy_05.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_06 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_06"

gen_test_data"policy_06"
thm policy_06.test_data


test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_07 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_07"

gen_test_data"policy_07"
thm policy_07.test_data
thm policy_07.pos

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_08 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_08"

gen_test_data"policy_08"
thm policy_08.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_09 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_09"

gen_test_data"policy_09"
thm policy_09.test_data

test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_10 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_10"

gen_test_data"policy_10"
thm policy_10.test_data





test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_11 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_11"

gen_test_data"policy_11"
thm policy_11.test_data




test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_12 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_12"

gen_test_data"policy_12"
thm policy_12.test_data




test_spec"not_in_same_net x \<longrightarrow> FUT x = policy_13 x"
apply (prepare_fw_spec)
apply (simp add: UnfoldPolicy PortCombinators PolicyCombinators not_in_same_net_def)
apply (gen_test_cases"FUT")
apply (simp_all add: not_in_same_net_def UnfoldPolicy)
store_test_thm"policy_13"

gen_test_data"policy_13"
thm policy_13.test_data


write_clocks "time_stats_rulesIntegerNetwork.tex"

declare [[testgen_profiling = false]]

end
