
theory 
  Scenario1_executable
imports 
  FWTesting
begin


subsubsection{* Network Model *}

text{* As there are only two distinct hosts in this model, we employ
the datatype address representation. The following definitions always
have to be provided when using the datatype representation, however
they are always standard (i.e. the same). The distribution already
provides the definitions for a datatype network consisting of three
elements. For the sake of completeness, we will not reuse that one
here.  *}

datatype hosts = Host1 | Host2

instantiation hosts :: enum
begin

definition enum_hosts_def     : "enum_class.enum = [Host1,Host2]"
definition enum_all_hosts_def : "enum_class.enum_all P = (P Host1 \<and>  P Host2)"
definition enum_ex_hosts_def  : "enum_class.enum_ex P = (P Host1 \<or>  P Host2)"

instance proof 
  show "UNIV = set (enum_class.enum \<Colon> hosts list)" 
       unfolding enum_hosts_def by(auto,case_tac "x", auto)
next
  show "distinct (enum_class.enum  \<Colon> hosts list)" 
       unfolding enum_hosts_def by auto
next
  fix P
  show "enum_class.enum_all (P :: hosts \<Rightarrow> bool) = (\<forall>x::hosts. P x)" 
       unfolding enum_all_hosts_def enum_hosts_def by(auto,case_tac "x", auto)
next
  fix P
  show "enum_class.enum_ex (P :: hosts \<Rightarrow> bool) = (\<exists>x::hosts. P x)" 
       unfolding enum_ex_hosts_def enum_hosts_def by(auto,case_tac "x", auto)
qed
end

type_synonym HostNets = "hosts \<times> port \<times> protocol"

instance hosts :: adr ..

defs(overloaded) 
 src_port_def[code_unfold]: "src_port (x::(HostNets,'b) packet) \<equiv> fst (snd (src x))"

 dest_port_def[code_unfold]: "dest_port (x::(HostNets,'b) packet) \<equiv> fst (snd (dest x))"

 src_protocol_def[code_unfold]: "src_protocol (x::(HostNets,'b) packet) \<equiv> snd (snd (src x))"

 dest_protocol_def[code_unfold]: "dest_protocol (x::(HostNets,'b) packet) \<equiv> snd (snd (dest x))"


definition host1 :: "HostNets set" 
where     "host1 = {(a,b). a = Host1}"


definition host2 :: "HostNets set" 
where     "host2 = {(a,b). a = Host2}"


subsubsection{* Policy Model *}

text{* We allow a couple of TCP and UDP ports in each direction and
deny everything else:*}
thm allow_from_to_ports_prot_def
definition
 TestPolicy ::"(HostNets,DummyContent)packet \<mapsto> unit" where
"TestPolicy =  
             (allow_from_to_ports_prot tcp {(1024::port)..4096}  {host1} {host2} \<Oplus> 
              allow_from_to_port_prot tcp (3389::port)    {host1} {host2} \<Oplus>
              allow_from_to_port_prot tcp (8080::port)    {host1} {host2} \<Oplus>
              allow_from_to_port_prot udp (8082::port)    {host2} {host1} \<Oplus>
              allow_from_to_port_prot tcp (21::port)      {host2} {host1} \<Oplus>
              allow_from_to_port_prot tcp (443::port)     {host2} {host1} \<Oplus> 
              allow_from_to_port_prot udp (22::port)      {host1} {host2} \<Oplus> 
              allow_from_to_port_prot tcp (23::port)      {host1} {host2} \<Oplus>        
              deny_all)"


subsubsection{* Excursion: Simulation Experiments via the Code Generator. *}

text{* Isabelle/ISAR allows for access of the underyling SML environment, 
where functions may be defined:*}
ML{*
fun fac 0 = 1
   |fac n = n * (fac (n -1)) 
*}
text{* ... and later on executed:*}
ML{* fac 10*}
text{* User-programmed functions may even refer to the internal Isabelle data-structures 
(the ML typesystem and Isabelle's Kernel Architecture will assure logical consistency) : *}
ML{* open Term;
     open Thm *}
text{* ... and logical inference operations:*}
ML{* sym RS sym*}

text{* So-called "antiquotations" make it even possible to express links between
the model and the SML level:*}
thm add_commute
ML{* @{thm add_commute} RS sym*}

text{* This may give you an idea how HOL-TestGen is internally built, and
why its implementation is actually "logically correct by construction".  *}

text{* A particular anti-quotation allows the user to invoke the code-generator
for (executable) logical HOL-expressions. Their values may be bound to variables
in the ML execution environment and used for computations: *}
ML{* val one = @{code "1::nat"}
     val suc = @{code "Suc"}
     val two = suc one*}

text{* Note that the current code generator version (Isabelle 2011-1) does not support
other input than constant symbols; synonyms, abbreviations etc. must be bound to a
constant symbol before:*}

definition "mt = Map.empty"  (* which is an abbreviation and is therefore no legal
                                argument in a code annotation. *)
definition "upd f a b = (f(a:= Some b))" (* the same *)

definition "lookup f x = f x"

definition "override f g = f \<Oplus> g"


text{* Now we can bind the entire infra-structure of partial maps ('data dictionaries') to
ML operators and execute them there. *}

ML{*
val mt = @{code mt}
fun upd f (x:''a) y = @{code upd}{equal = fn x => fn y => x=y} f x y
val dom = @{code dom}
val ran = @{code ran}
val lookup = @{code lookup}
val override = @{code "override"}
*}

text{*which allows us to do things like:*}

ML{* 
val A = upd(upd mt 1 1) 2 2
val B = upd mt 3 4
val Y = override A B
val Z = lookup Y 2
*}


subsubsection{* Transition from Policies to Executable Policies: *}

text{* A direct attempt to compile \verb+TestPolicy+ to code via \verb+{code "TestPolicy"}+
fails; the compiler reports a Wellsortedness error:
\begin{verbatim}
Type hosts \<times> int \<times> protocol not of sort {enum,equal,adr}
No type arity int :: enum 
\end{verbatim}
A more high-level, semantic explanation of this error is that the combinator definitions use 
internally sets of sets of HostNets, which require the latter to be enumerable in order to have
a means to constructively decide the equality required by the outermost notion of "set".
One can, of course, bound the port-adresses which would repair this effect. Still, the
resulting code would be hopelessly inefficient.
*}

text{* For our purpose of code-generation, we provide therefore a specialized definition
for \verb+allow_from_to_port_prot+, which has the advantage to be executable: *}

definition allow_from_to_port_prot_single :: 
          "protocol \<Rightarrow> port \<Rightarrow> HostNets set \<Rightarrow> HostNets set \<Rightarrow> (HostNets,DummyContent)packet \<mapsto> unit" 
where     "allow_from_to_port_prot_single pr po sr de =
                {pa. src pa \<in> sr \<and> dest pa \<in> de \<and> dest_protocol pa = pr \<and> dest_port pa = po}  \<triangleleft> A\<^sub>U"

definition allow_from_to_ports_prot_single :: 
       "protocol \<Rightarrow> port set \<Rightarrow> HostNets set \<Rightarrow> HostNets set \<Rightarrow> (HostNets,DummyContent)packet \<mapsto> unit" 
where "allow_from_to_ports_prot_single pr po sr de =
            {pa. src pa \<in> sr \<and> dest pa \<in> de \<and> dest_protocol pa = pr \<and> dest_port pa \<in> po}  \<triangleleft> A\<^sub>U"

text {* Here is the proof that the executable definition and the original one are equivalent
for the special case of being instatiated with singleton sets. We configure the code-generator 
from this fact, which uses it in a pre-processing phase by rewriting the policy to 
be compiled:  *}

lemma allow_from_to_port_prot2exec[code_unfold]: 
"allow_from_to_port_prot  pr po {sr} {de} = allow_from_to_port_prot_single pr po sr de"
by(auto simp: ProtocolCombinators allow_from_to_port_prot_single_def in_subnet_def )

lemma allow_from_to_ports_prot2exec[code_unfold]: 
"allow_from_to_ports_prot  pr po {sr} {de} = allow_from_to_ports_prot_single pr po sr de"
by(auto simp: ProtocolCombinators allow_from_to_ports_prot_single_def in_subnet_def)

text{* We can now rewrite the original definition to an executable --- and thus compilable ---
form: *}
thm TestPolicy_def[simplified allow_from_to_port_prot2exec allow_from_to_ports_prot2exec]


text{* Now we have everything together to use the code generator to directly generate code
from our policy-definitions: Be it for animations, or even for reference implementations
(since this code can be exported and executed independently from Isabelle using the
MLton compiler, for example). *}

ML{* 
val zero = @{code "0::int"}
val one = @{code "1::int"}
val add_inta = @{code "plus::int\<Rightarrow>int\<Rightarrow>int"}
val mult_inta = @{code "times::int\<Rightarrow>int\<Rightarrow>int"}
val two = add_inta one one;

fun conv_inta 0 = @{code "0::int"}
   |conv_inta 1 = @{code "1::int"}
   |conv_inta x = (let val C = mult_inta two (conv_inta(x div 2)) 
                   in if x mod 2 = 0 then C
                      else add_inta one C
                   end)

val HOST1 = @{code "Host1"}
val HOST2 = @{code "Host2"}
val TCP = @{code "tcp"}
val UDP = @{code "udp"}

val res = @{code "TestPolicy"} (conv_inta 134,
                                ((HOST1,(conv_inta 100,TCP)),
                                 ((HOST1,(conv_inta 120,UDP)),
                                @{code "data"})))

*}

text{*... or, for short: *}

value "TestPolicy (134,((Host1,(100,tcp)), ((Host2,(120,udp)), data)))"
value "TestPolicy (134,((Host2,(100,tcp)), ((Host1,(21,tcp)), data)))"


text{* In order to generate standalone-code, the following sequence of commands does the trick: *}

(* code_type int (SML "int") ... how could this be done to improve the code ??? *) 

export_code TestPolicy  in SML
module_name Scenario file "Scenario1.ML"

text{* This operation generates the following SML code : 

\begin{verbatim}
structure Scenario : sig
  datatype inta = Pls | Min | Bit0 of inta | Bit1 of inta |
    Number_of_int of inta
  val member : 'a -> ('a -> bool) -> bool
  val map_add : ('a -> 'b option) -> ('a -> 'b option) -> 'a -> 'b option
  val collect : ('a -> bool) -> 'a -> bool
  datatype 'a decision = Allow of 'a | Deny of 'a
  val equal_int : inta -> inta -> bool
  type 'a adr
  val fst : 'a * 'b -> 'a
  val snd : 'a * 'b -> 'b
  val src : 'a adr -> inta * ('a * ('a * 'b)) -> 'a
  datatype protocol = Tcp | Udp
  datatype dummyContent = Data
  val dest : 'a adr -> inta * ('a * ('a * 'b)) -> 'a
  val adr_int : inta adr
  val adr_prod : 'a adr -> 'b adr -> ('a * 'b) adr
  datatype hosts = Host1 | Host2
  val equal_protocol : protocol -> protocol -> bool
  val deny_all : 'a -> unit decision option
  val equal_hosts : hosts -> hosts -> bool
  val host1a : hosts * (inta * protocol) -> bool
  val host2a : hosts * (inta * protocol) -> bool
  val allow_all : 'a -> unit decision option
  val adr_hosts : hosts adr
  val adr_protocol : protocol adr
  val dom_restrict : ('a -> bool) -> ('a -> 'b decision option) -> 'a -> 'b decision option
  val allow_from_to_port_prot_single :
    protocol -> inta -> (hosts * (inta * protocol) -> bool) -> (hosts * (inta * protocol) -> bool)->
    inta * ((hosts * (inta * protocol)) * ((hosts * (inta * protocol)) * dummyContent)) ->
    unit decision option
  val testPolicy :
    inta * ((hosts * (inta * protocol)) *((hosts * (inta * protocol)) * dummyContent)) ->
      unit decision option
end = struct ... end
\end{verbatim}
*}

end