theory 
  Scenario2
imports 
  FWTesting
begin

subsubsection{* Network Model *}

text{* We model the network addresses as Integer: all address in
subnetwork 1 get an address between 2 and 255, all in subnetwork 2
between 1002 and 1255. We omit 1 and 1001 for the simple reason that
this will often be mapped to the address of the test execution manager
- we avoid any possible problems due to this. *}

definition adrs1 :: "int set" where "adrs1 = {x. x > 1 \<and> x < 256}"
definition adrs2 :: "int set" where "adrs2 = {x. x > 1001 \<and> x < 1256}"

definition
 subnet1 :: "adr\<^isub>i\<^isub>p\<^isub>p net" where
"subnet1 = {{(a,b). a \<in> adrs1}}"

definition
 subnet2 :: "adr\<^isub>i\<^isub>p\<^isub>p net" where
"subnet2 = {{(a,b). a \<in> adrs2}}"


subsubsection{* Policy Model *} 

text{* The policy is exactly the same as before - except that the
allowed ports are between the two networks instead of the single hosts
only. Please note also that these two policies are of a different type
as well. *}

definition
 TestPolicy  where
"TestPolicy =  
             (allow_from_to_ports_prot tcp {(1024::port)..4096}  subnet1 subnet2 \<Oplus>
              allow_from_to_port_prot tcp (3389::port)           subnet1 subnet2 \<Oplus>
              allow_from_to_port_prot tcp (8080::port)           subnet1 subnet2 \<Oplus>
              allow_from_to_port_prot udp (8082::port)           subnet1 subnet2 \<Oplus>
              allow_from_to_port_prot tcp (21::port)             subnet1 subnet2 \<Oplus>
              allow_from_to_port_prot tcp (443::port)            subnet1 subnet2 \<Oplus>
              allow_from_to_port_prot udp (22::port)             subnet1 subnet2 \<Oplus>
              allow_from_to_port_prot tcp (23::port)             subnet1 subnet2 \<Oplus>       
              deny_all)"

subsubsection{* Definitions of the testing constraints. *} 

text{* We have the same three testing constraints as before. However,
this time, we can re-use the standard definitions for
\texttt{port\_positive} and \texttt{fix\_values}, as they are included
in the HOL-TestGenFW library for all default address models.  *}

definition
 accross_subnets :: "(adr\<^isub>i\<^isub>p\<^isub>p,'b) packet \<Rightarrow> bool" where
"accross_subnets x = (((src x \<sqsubset> subnet1) \<and> (dest x \<sqsubset> subnet2)) \<or>  
                     ((src x \<sqsubset> subnet2)  \<and> (dest x \<sqsubset> subnet1)))" 


subsubsection{* Test Generation *}

text{* Following the standard procedures, all required lemmas are put
into a group lemma: *} 

lemmas policyLemmas = TestPolicy_def accross_subnets_def 
  ProtocolPortCombinators.ProtocolCombinators 
  adrs1_def adrs2_def subnet1_def subnet2_def 
  adr\<^isub>i\<^isub>p\<^isub>pLemmas adr\<^isub>i\<^isub>p\<^isub>pTestConstraints
  packet_defs



text{* The test specification and its processing is again the same as before: *}
test_spec 
"port_positive x \<and> accross_subnets x \<and> fix_values x  \<longrightarrow> 
FUT x = TestPolicy x"
apply  (prepare_fw_spec)
apply (simp add: policyLemmas)
apply (gen_test_cases "FUT")
apply (simp_all add: policyLemmas)
store_test_thm "Scenario2"

declare[[testgen_SMT=true]]
declare [[testgen_iterations=5]]

gen_test_data "Scenario2"
text {* 60 test data are generated, also this is the same number as
before. *}
export_fw_data "Scenario2.data" Scenario2
end