(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: ethz-integer-base.thy 9413 2012-10-07 18:52:48Z lukasbru $ *)

theory 
  "ethz-integer-base"
imports 
  FWTesting
begin

declare [[testgen_profiling]]


(* The subnetworks *)

(* 
 Mapping to the subnetworks

 129.132.240.0/26     \<Longrightarrow>  1
 129.132.176.160/27   \<Longrightarrow>  2
 FW-LINK              \<Longrightarrow>  3
 129.132.254.33/32    \<Longrightarrow>  4
 224.0.0.0/4          \<Longrightarrow>  5
 SIP-Server           \<Longrightarrow>  6
 time.ethz.ch         \<Longrightarrow>  7
 DHCP                 \<Longrightarrow>  8
 129.132.105.4/32     \<Longrightarrow>  9
 129.132.105.10/32    \<Longrightarrow>  10

 ETHZ-Intern          \<Longrightarrow>  11
 ETHZ-Private         \<Longrightarrow>  12
 VPN-VPZ              \<Longrightarrow>  13


*)

definition
 FWLink :: "adr\<^isub>i\<^isub>p net" where
"FWLink = {{(a,b). a = 3}}"

definition
 any :: "adr\<^isub>i\<^isub>p net" where
"any = {{(a,b). a > 13}}"

definition
 i26:: "adr\<^isub>i\<^isub>p net" where
"i26 = {{(a,b). a = 1 }}"

definition
 i27:: "adr\<^isub>i\<^isub>p net" where
"i27 = {{(a,b). a = 2 }}"

definition
 i32:: "adr\<^isub>i\<^isub>p net" where
"i32 = {{(a,b). a =  4}}"

definition
 i4:: "adr\<^isub>i\<^isub>p net" where
"i4 = {{(a,b). a = 5 }}"

definition
 sip:: "adr\<^isub>i\<^isub>p net" where
"sip = {{(a,b). a = 6 }}"

definition
 time:: "adr\<^isub>i\<^isub>p net" where
"time = {{(a,b). a = 7}}"

definition
 dhcp:: "adr\<^isub>i\<^isub>p net" where
"dhcp = {{(a,b). a =  8}}"
 
definition
 i4_32:: "adr\<^isub>i\<^isub>p net" where
"i4_32 = {{(a,b). a =  9}}"

definition
 i10_32:: "adr\<^isub>i\<^isub>p net" where
"i10_32 = {{(a,b). a = 10 }}"

definition
 eth_intern:: "adr\<^isub>i\<^isub>p net" where
"eth_intern = {{(a,b). a = 11 }}"
 
definition
 eth_private:: "adr\<^isub>i\<^isub>p net" where
"eth_private = {{(a,b). a = 12 }}"

definition
 vpn_vpz :: "adr\<^isub>i\<^isub>p net" where
"vpn_vpz = {{(a,b). a = 13}}"


definition
(* Mandatory: Global *)

 MG1 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG1 = allow_from_to_port (1::port) i26 any"

definition
 MG2 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG2 = allow_from_to_ports {(1::port),(2::port),(4::port)} i27 any"

definition
 MG3 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG3 = allow_from_to_port (1::port) any FWLink"

definition
 MG4 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG4 = allow_from_to_port (3::port) FWLink FWLink
"
definition
 MG5 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG5 = allow_from_to_port (5::port) i32 any"

definition
 MG6 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG6 = allow_from_to_port (6::port) i32 i4 "
 
definition
 MG7 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG7 = allow_from_to_ports {(6::port),(7::port)} FWLink i4"

definition
 MG8 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG8 = allow_from_to_ports {(6::port),(7::port)} FWLink i4"

definition
 MG9 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG9 = allow_from_to_port (5::port) any i32 "

definition
 MG10 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG10 = allow_from_to_port (5::port) i32 any"

definition
 MG11 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG11 = allow_from_to_port (1::port) FWLink any"

definition
 MG12 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG12 = allow_from_to_port (3::port) FWLink FWLink"

definition
 MG13 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MG13 = allow_from_to_ports {(4::port), (2::port)} sip any "

(* Default Global *)

definition
 DG1 :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"DG1 = allow_from_to_port (8::port) time any "

definition
 DG2:: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"DG2 = allow_from_to_port (1::port) dhcp any "

definition
 DG3:: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"DG3 = deny_from_to_port (7::port) any any "

 


definition 
 MandatoryGlobalPolicy :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MandatoryGlobalPolicy = MG13 ++ MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8 ++  MG7 ++  MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"   
      
definition
 MandatoryGlobalInsidePolicy :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MandatoryGlobalInsidePolicy = MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8"
         
definition
 MandatoryGlobalOutsidePolicy :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"MandatoryGlobalOutsidePolicy =   MG13 ++ MG7 ++ MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"       

definition
 DefaultGlobalPolicy :: "(adr\<^isub>i\<^isub>p,DummyContent) FWPolicy" where
"DefaultGlobalPolicy = DG3 ++ DG2 ++ DG1"  

 lemmas PolicyLemmas = adr\<^isub>i\<^isub>pLemmas   
 src_def dest_def content_def in_subnet_def 
 FWLink_def
 any_def
 i26_def
 i27_def
 i32_def
 i4_def
 sip_def
 time_def
 dhcp_def
 i4_32_def
 i10_32_def
 eth_intern_def
 eth_private_def
 vpn_vpz_def
 MG1_def MG2_def MG3_def MG4_def MG5_def MG6_def MG7_def MG8_def MG9_def MG10_def MG11_def MG12_def MG13_def
 DG1_def DG2_def DG3_def
 MandatoryGlobalPolicy_def MandatoryGlobalOutsidePolicy_def MandatoryGlobalInsidePolicy_def DefaultGlobalPolicy_def

definition 
 not_in_same_net :: "(adr\<^isub>i\<^isub>p,DummyContent) packet \<Rightarrow> bool" where
"not_in_same_net x =  (((src x \<sqsubset> i26 ) \<longrightarrow> ( \<not> (dest x \<sqsubset> i26 ))) \<and>  
                      ((src x \<sqsubset> i27) \<longrightarrow> ( \<not> (dest x \<sqsubset> i27))) \<and>
                      ((src x \<sqsubset> i32 ) \<longrightarrow> ( \<not> (dest x \<sqsubset> i32))) \<and>
                      ((src x \<sqsubset>  i4) \<longrightarrow> ( \<not> (dest x \<sqsubset> i4))) \<and>
                      ((src x \<sqsubset>  sip) \<longrightarrow> ( \<not> (dest x \<sqsubset> sip))) \<and>
                      ((src x \<sqsubset>  time) \<longrightarrow> ( \<not> (dest x \<sqsubset> time))) \<and>
                      ((src x \<sqsubset>  dhcp) \<longrightarrow> ( \<not> (dest x \<sqsubset> dhcp))) \<and>
                      ((src x \<sqsubset>  i4_32) \<longrightarrow> ( \<not> (dest x \<sqsubset> i4_32))) \<and>
                      ((src x \<sqsubset>  i10_32) \<longrightarrow> ( \<not> (dest x \<sqsubset> i10_32))) \<and>
                      ((src x \<sqsubset>  eth_intern) \<longrightarrow> ( \<not> (dest x \<sqsubset> eth_intern))) \<and>
                      ((src x \<sqsubset>  vpn_vpz) \<longrightarrow> ( \<not> (dest x \<sqsubset> vpn_vpz))) \<and>
                      ((src x \<sqsubset>  eth_private) \<longrightarrow> ( \<not> (dest x \<sqsubset> eth_private))))"
                   


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas  PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "DG"

gen_test_data "DG"
thm DG.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalInsidePolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas   PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "MGI"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalOutsidePolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas   PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "MGO"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas   PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "MG"


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy ++ MandatoryGLobalPolicy) x"
apply (prepare_fw_spec)
apply (simp add: PolicyLemmas  PortCombinators PolicyCombinators PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas ) 
store_test_thm "all"


write_clocks "time_stats_ethz_integer_base"


end
