theory FTPVOIPTest
imports 
  FWTesting
begin

definition 
  intranet :: "adr\<^isub>i\<^isub>p net" where
  "intranet = {{(a,e) . a = 3}}"

definition 
  internet :: "adr\<^isub>i\<^isub>p net" where
  "internet = {{(a,c). a > 4}}"

definition 
  gatekeeper :: "adr\<^isub>i\<^isub>p net" where
  "gatekeeper = {{(a,c). a =4}}"


definition 
  voip_policy :: "(adr\<^isub>i\<^isub>p, ftpvoip) FWPolicy" where
  "voip_policy = A\<^sub>U"



definition 
  is_in_intranet :: "address  \<Rightarrow> bool" where
  "is_in_intranet a = (a = 3)"

definition 
  is_gatekeeper :: "address \<Rightarrow> bool" where
  "is_gatekeeper a =  (a = 4)"

definition 
  is_in_internet :: "address \<Rightarrow> bool" where
  "is_in_internet a =  (a > 4)"



definition 
  "\<sigma>_0_voip" ::  "(adr\<^isub>i\<^isub>p, ftpvoip) history \<times>
                 (adr\<^isub>i\<^isub>p,  ftpvoip) FWPolicy" 
where                                     
  "\<sigma>_0_voip = ([],voip_policy)"

definition "accept_ftpvoip" ::  "id \<Rightarrow> id \<Rightarrow> (adr\<^isub>i\<^isub>p,  ftpvoip) history \<Rightarrow> bool" where
          "accept_ftpvoip i1 i2 t = 
  (\<exists> s1 d1 p1 vs vd vg vp1 vp2. t \<in> ftp_voip_interleaved s1 d1 i1 p1 vs vd vg i2 vp1 vp2
 \<and> is_in_intranet vs \<and> is_in_internet vd \<and> is_gatekeeper vg \<and>
   is_in_intranet (fst s1) \<and> is_in_internet (fst d1) \<and> (snd d1) = 21 \<and>
            i1 \<noteq> i2)"
                 



declare [[testgen_profiling]]




fun ids1 where
 "ids1 i (x#xs) = (id x = i \<and> ids1 i xs)"
|"ids1 i [] = True"

lemmas ST_simps2 = FTPVOIP_ST_simps Let_def valid_SE_def unit_SE_def bind_SE_def
 subnet_of_def port_open_def is_init_def 
  p_accept_def content_def 
 is_in_intranet_def is_in_internet_def intranet_def internet_def exI
 subnetOf_lemma subnetOf_lemma2 subnetOf_lemma3 subnetOf_lemma4
 port_open_lemma voip_policy_def
 NetworkCore.id_def  is_arq_def is_fin_def
 is_connect_def is_setup_def ports_open_def subnet_of_adr_def
 NB_voip_def \<sigma>_0_voip_def  PLemmas VOIP_TRPolicy_def
 policy2MON_def applyPolicy_def


declare [[testgen_depth=5]]

test_spec "\<lbrakk> length t = length X; accept_ftpvoip 1 2 (rev t); ids {1,2} t; length t <= 5\<rbrakk> \<Longrightarrow>
 (\<sigma>_0_voip \<Turnstile> (os \<leftarrow> mbind t FTPVOIP_TRPolicy; return (os=X))) \<longrightarrow>
  FUT \<sigma>_0_voip t = X"
apply(simp add: accept_ftpvoip_def \<sigma>_0_voip_def)
apply (rule impI)+
apply (unfold  ftp_voip_interleaved_def is_in_internet_def is_in_intranet_def is_gatekeeper_def)
apply (simp)
apply (gen_test_cases "FUT" simp:id_def )
apply (simp_all)
store_test_thm "voip_test5"


declare [[testgen_depth=6]]

test_spec "\<lbrakk> length t = length X; accept_ftpvoip 1 2 (rev t); ids {1,2} t; length t <= 6\<rbrakk> \<Longrightarrow>
 (\<sigma>_0_voip  \<Turnstile> (os \<leftarrow> mbind t FTPVOIP_TRPolicy; return (os=X))) \<longrightarrow>
  FUT \<sigma>_0_voip t = X"
apply(simp add: accept_ftpvoip_def \<sigma>_0_voip_def)
apply (rule impI)+
apply (unfold  ftp_voip_interleaved_def is_in_internet_def is_in_intranet_def is_gatekeeper_def)
apply (simp)
apply (gen_test_cases "FUT" simp:id_def )
apply (simp_all)
store_test_thm "voip_test6"

declare [[testgen_depth=7]]

test_spec "\<lbrakk> length t = length X; accept_ftpvoip 1 2 (rev t); ids {1,2} t; length t <= 7\<rbrakk> \<Longrightarrow>
 (\<sigma>_0_voip  \<Turnstile> (os \<leftarrow> mbind t FTPVOIP_TRPolicy; return (os=X))) \<longrightarrow>
  FUT \<sigma>_0_voip t = X"
apply(simp add: accept_ftpvoip_def \<sigma>_0_voip_def)
apply (rule impI)+
apply (unfold  ftp_voip_interleaved_def is_in_internet_def is_in_intranet_def is_gatekeeper_def)
apply (simp)
apply (gen_test_cases "FUT" simp:id_def )
apply (simp_all)
store_test_thm "voip_test7"

declare [[testgen_depth=8]]

test_spec "\<lbrakk> length t = length X; accept_ftpvoip 1 2 (rev t); ids {1,2} t; length t <= 8\<rbrakk> \<Longrightarrow>
 (\<sigma>_0_voip  \<Turnstile> (os \<leftarrow> mbind t FTPVOIP_TRPolicy; return (os=X))) \<longrightarrow>
  FUT \<sigma>_0_voip t = X"
apply(simp add: accept_ftpvoip_def \<sigma>_0_voip_def)
apply (rule impI)+
apply (unfold  ftp_voip_interleaved_def is_in_internet_def is_in_intranet_def is_gatekeeper_def)
apply (simp)
apply (gen_test_cases "FUT" simp:id_def )
apply (simp_all)
store_test_thm "voip_test8"

declare [[testgen_depth=9]]

test_spec "\<lbrakk> length t = length X; accept_ftpvoip 1 2 (rev t); ids {1,2} t; length t <= 9\<rbrakk> \<Longrightarrow>
 (\<sigma>_0_voip  \<Turnstile> (os \<leftarrow> mbind t FTPVOIP_TRPolicy; return (os=X))) \<longrightarrow>
  FUT \<sigma>_0_voip t = X"
apply(simp add: accept_ftpvoip_def \<sigma>_0_voip_def)
apply (rule impI)+
apply (unfold  ftp_voip_interleaved_def is_in_internet_def is_in_intranet_def is_gatekeeper_def)
apply (simp)
apply (gen_test_cases "FUT" simp:id_def )
apply (simp_all)
store_test_thm "voip_test9"

declare [[testgen_depth=10]]

test_spec "\<lbrakk> length t = length X; accept_ftpvoip 1 2 (rev t); ids {1,2} t; length t <= 10\<rbrakk> \<Longrightarrow>
 (\<sigma>_0_voip  \<Turnstile> (os \<leftarrow> mbind t FTPVOIP_TRPolicy; return (os=X))) \<longrightarrow>
  FUT \<sigma>_0_voip t = X"
apply(simp add: accept_ftpvoip_def \<sigma>_0_voip_def)
apply (rule impI)+
apply (unfold  ftp_voip_interleaved_def is_in_internet_def is_in_intranet_def is_gatekeeper_def)
apply (simp)
apply (gen_test_cases "FUT" simp:id_def )
apply (simp_all)
store_test_thm "voip_test10"


end
