#!/bin/bash

 echo "
 
header{* File generated by script \"generate_rule_thy.sh\", on 
`date`.
*}

theory 
  fwrules
imports
  FWTesting
begin

declare [[testgen_profiling]]
declare [[testgen_iterations=100]]
declare [[testgen_SMT=true]]

definition
  workstation::\"adr\<^isub>i\<^isub>p net\" where
  \"workstation = {{(a,b) . (a=3) }}\"
definition
  internet :: \"adr\<^isub>i\<^isub>p net\" where
  \"internet = {{(a,b).  \<not> (a =3)}}\"

definition
  not_in_same_net :: \"(adr\<^isub>i\<^isub>p,DummyContent) packet \<Rightarrow> bool\" where
  \"not_in_same_net x  =  ((src x \<sqsubset> workstation \<longrightarrow> ( (dest x \<sqsubset> internet))) \<and> 
  (src x \<sqsubset> internet \<longrightarrow>  dest x \<sqsubset> workstation))\"

definition
   policy_0 :: \"(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy\" where
  \"policy_0 = deny_all\"

definition \"policy_0_Summary  = policy_0\"

lemmas UnfoldPolicy0 =  src_def dest_def in_subnet_def IntegerPort.src_port_def 
                       IntegerPort.dest_port_def 
                       workstation_def internet_def policy_0_def 
                       
lemmas UnfoldPolicyS0 = policy_0_Summary_def
test_spec \"not_in_same_net x \<longrightarrow> FUT x = policy_0 x\"
apply (prepare_fw_spec)
apply (gen_test_cases \"FUT\" simp: UnfoldPolicy0 PortCombinators)
apply (simp_all add: not_in_same_net_def UnfoldPolicy0)
store_test_thm \"policy_0\"

gen_test_data \"policy_0\"

" > fwrules.thy
test=10
for ((i=1; i<=500; i++))

do


  j=$i
  let "j = $j - 1"
  k="policy_${i}_def"

echo "
definition
   policy_$i :: \"(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy\" where
  \"policy_$i = policy_$j ++ allow_from_to_port ($i::port) workstation internet\"

lemmas UnfoldPolicy$i = UnfoldPolicy$j $k 
" >> fwrules.thy



if test $i -eq $test
then
let "test = $test + 10"
  let "former = $i - 10"
  let "formerP = $i - 9"
  f="policy_${i}_Summary_def"

echo "


definition
   policy_${i}_Summary :: \"(adr\<^isub>i\<^isub>p, DummyContent) FWPolicy\" where
  \"policy_${i}_Summary = policy_${former}_Summary ++ allow_from_to_ports {$formerP..($i::port)} workstation internet\"

lemmas UnfoldPolicyS$i = UnfoldPolicyS$former $f

" >> fwrules.thy


fi

done


for i in 10 20 30 40 50 60 70 80 90 100 120 140 160 180 200 250 
do



echo "
test_spec \"not_in_same_net x \<longrightarrow> FUT x = policy_$i x\"
apply (prepare_fw_spec)
apply (gen_test_cases \"FUT\" simp:  UnfoldPolicy$i PortCombinators not_in_same_net_def)
apply (simp_all add: UnfoldPolicy$i)
store_test_thm \"policy_$i\"

gen_test_data \"policy_$i\" 
" >> fwrules.thy

done


for i in 10 20 30 40 50 60 70 80 90 100 120 140 
do

echo "
test_spec \"not_in_same_net x \<longrightarrow> FUT x = policy_${i}_Summary x\"
apply (prepare_fw_spec)
apply (gen_test_cases \"FUT\" simp: UnfoldPolicyS$i  UnfoldPolicy$i PortCombinators not_in_same_net_def)
apply (simp_all add: UnfoldPolicy$i UnfoldPolicyS$i)
store_test_thm \"policy_${i}_Summary\"

gen_test_data \"policy_${i}_Summary\" 

" >> fwrules.thy

done


echo "

write_clocks \"time_stats_fwrules.tex\"

declare [[testgen_profiling = false]]

end" >> fwrules.thy
