(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * SimpleDMZIntegerDocument.thy --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2010 Université Paris-Sud, France*
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: SimpleDMZIntegerDocument.thy 9105 2011-01-25 22:03:54Z makarius $ *)

theory 
  SimpleDMZIntegerDocument
imports 
  FWTesting
begin

text{*

This is a typical example for a small stateless packet filter. There
are three subnetworks, with either none or some protocols allowed
between them.

We use IntegerPort as the address model.
*}

definition
  intranet::"IntegerPort net" where
  "intranet = {{(a,b) . a = 3}}"

definition
  dmz :: "IntegerPort net" where
  "dmz = {{(a,b). a = 7}}"

definition
  internet :: "IntegerPort net" where
  "internet = {{(a,b).  \<not> (a=3 \<or> a =7)}}"


definition
  Intranet_DMZ_Port :: "(IntegerPort,DummyContent) FWPolicy" where
  "Intranet_DMZ_Port = allow_from_to_port ftp intranet dmz"

definition
  Intranet_Internet_Port :: "(IntegerPort,DummyContent) FWPolicy" where
  "Intranet_Internet_Port = allow_from_to_port http intranet internet"

definition           
  Internet_DMZ_Port :: "(IntegerPort,DummyContent) FWPolicy" where
  "Internet_DMZ_Port = allow_from_to_port smtp internet dmz"


text{*
The policy: *}
definition  policy :: "(IntegerPort, DummyContent) FWPolicy" where
 "policy =       ( (Intranet_Internet_Port \<Oplus>
                  Intranet_DMZ_Port) \<Oplus> 
                  Internet_DMZ_Port )\<Oplus> deny_all"


lemmas PolicyLemmas =  dmz_def internet_def intranet_def
                       Intranet_Internet_Port_def Intranet_DMZ_Port_def 
                       Internet_DMZ_Port_def policy_def
                       src_def dest_def in_subnet_def
                       IntegerPortLemmas
                       content_def

text{*
Only create test cases crossing network boundaries.
*}
definition not_in_same_net :: "(IntegerPort,DummyContent) packet \<Rightarrow> bool" where
 "not_in_same_net x  =  ((src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                     (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                     (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz))"

 

declare Ports [simp add]

text{* The test specification: *}
test_spec "not_in_same_net x \<longrightarrow> FUT x = policy x"
  apply (prepare_fw_spec)
  apply (simp add: not_in_same_net_def PolicyLemmas PortCombinators
                  PolicyCombinators)
  apply (gen_test_cases "FUT")
  apply (simp_all add: PolicyLemmas)
store_test_thm "PolicyTest"


declare [[testgen_iterations=100]]

gen_test_data "PolicyTest"


text{*
The set of generated test data is:

@{thm "PolicyTest.test_data"}
*}
end
