(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * TestScript.thy --- a base testing theory for sequential computations.
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2011 Achim D. Brucker, Germany
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id:$ *)


theory 
TestScript
imports 
TestGen
(* Code_Integer *) 
begin 

datatype Lazy = "lazy" bool

code_const "lazy" (SML "fn () ==> (_)") 
(* code_const lazy (Scala "lazy ( _ )")  *)

code_type Lazy (SML "bool \<Rightarrow> bool")

ML{*

  fun gen_ts_def dataname lthy = let 
  val defname  = dataname^"_test_script" 
  fun get_td datname thy = (TestGen.get_test_data thy dataname,thy)

  val (td, lthy') = Local_Theory.raw_theory_result (get_td dataname) lthy 
  
  

  fun def bnd term lthy = let 
      val ((_, (_, thm)), lthy') = Local_Theory.define (((Binding.name bnd), NoSyn),
                    ((Binding.name "bnd",[Code.add_default_eqn_attrib]), term)) lthy
     in lthy' end 
                    
  
  val prop_terms = map (fn p => let val _ $ ret = concl_of p  in ret end) td;
  val prems_terms = map (fn data => (map (fn p => let val _ $ ret = p in ret end) 
                        (prems_of data))) td;
  val spec = map2 (fn x => fn y => (x,y))  prems_terms prop_terms

(*
  val absBoolT = HOLogic.unitT --> HOLogic.boolT
  fun mk_abs t = lambda (Free ("()",HOLogic.unitT)) t  
*)

  val absBoolT = HOLogic.boolT
  fun mk_abs t = Const("TestScript.Lazy.lazy",@{typ "bool"} --> @{typ "Lazy"})$t  


  val ts_term = HOLogic.mk_list (HOLogic.mk_tupleT[HOLogic.listT @{typ "Lazy"},@{typ "Lazy"}])
  (map (fn (a,b) =>  HOLogic.mk_tuple [HOLogic.mk_list @{typ "Lazy"} (map mk_abs a), mk_abs b]) spec) 


  in 
    def defname ts_term lthy'
  end

*}
ML {* 
  Outer_Syntax.command "prepare_ts_code" "prepare definitions for test script generation" Keyword.thy_decl 
            (Parse.name >> (fn name => Toplevel.local_theory NONE (gen_ts_def name)));
 *}




end
