(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * harness.sml --- the HOL-TestGen test harness
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2007 ETH Zurich, Switzerland
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: harness.sml 9315 2012-02-17 17:24:00Z brucker $ *)

signature TESTHARNESS = 
sig  
    type testresult
    val check : (unit -> 'a option) 
		-> (unit -> bool) list -> (unit -> bool)  
		-> ((bool option * exn option) * 
		    (bool option * 'a  option * exn option))
    val printResult : ('a -> string) -> int 
		      -> ((bool option * exn option) * 
			  (bool option * 'a option * exn option)) 
		      -> testresult
    val printList : ('a -> string) 
		    -> ((bool option * exn option) * 
			(bool option * 'a option * exn option)) list 
		    -> unit
    val rsf :bool
    val string_to_char : string -> char
end

structure TestHarness  : TESTHARNESS = 
struct
datatype testresult = SUCCESS | WARNING | FAILURE | ERROR | FATAL

val rsf = false;

fun init () = print "NOT YET IMPLEMENTED"
	      
fun check retval prelist testpred  = 
    let
	fun fst (x,y)= x;
	val pre = (SOME(List.foldl (fn (x,y) => x andalso y) true 
			      (map (fn x => (x () = true)) prelist)),NONE)
	    handle e => (NONE,SOME(e));	
	val post =  (SOME((testpred()=true)),retval(),NONE) 
	    handle e => (NONE,NONE,SOME(e));    
    in 
	(pre,post)
    end
    
fun printResultValue toString res = 
    (case res of
	 NONE   => "\n"
       | SOME r => ", result: "^(toString r)^"\n")
    
fun printResult toString num ((SOME(true),NONE),(SOME(true),res,NONE)) =   
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " -     SUCCESS"^
		       (printResultValue toString res))
    in 
	SUCCESS
    end
  | printResult toString num ((SOME(true),NONE),(SOME(false),res,NONE)) =  
    let
	val _ = print ("Test "^(Int.toString(num))
		       ^" - *** FAILURE: post-condition false"^
		       (printResultValue toString res))
    in 
	FAILURE
    end
  | printResult toString num ((SOME(true),NONE),(NONE,res,SOME(e_post))) = 
    let
	val _ = print ("Test "^(Int.toString(num))
		       ^" - *** FAILURE: exception during post-condition\n")
    in 
	FAILURE
    end
  | printResult toString num ((SOME(false),NONE),(SOME(true),res,NONE))=  
    let
	val _ = print ("Test "^(Int.toString(num))
		       ^" -     WARNING: pre-condition false "^
		       "(post-condition true)"^(printResultValue toString res))
    in 
	WARNING
    end
  | printResult toString num ((SOME(false),NONE),(SOME(false),res,NONE)) =
    let
	val _ = print ("Test "^(Int.toString(num))
		       ^" - *   WARNING: pre-condition false "^
		       "(post-condition false)"
		       ^(printResultValue toString res))
    in 
	WARNING
    end
  | printResult toString num ((SOME(false),NONE),(NONE,res,SOME(e_post))) =
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " - **  WARNING: pre-condition false "^
		       "(exception during post_condition)\n")
    in 
	WARNING
    end
  | printResult toString num ((NONE,SOME(e_pre)),(SOME(true),res,NONE)) =  
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " - **  ERROR:exception during pre-condition "^
		       "(post-condition true)"^(printResultValue toString res))
    in 
	ERROR
    end
  | printResult toString num ((NONE,SOME(e_pre)),(SOME(false),res,NONE)) = 
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " - **  ERROR: exception during pre-condition "^
	   "(post-condition false)"^(printResultValue toString res))
    in 
	ERROR
    end
  | printResult toString num ((NONE,SOME(e_pre)),(NONE,res,SOME(e_post))) =
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " - **  ERROR: exception during pre-condition "^
		       "and post-condition\n")
    in 
	ERROR
    end
  | printResult toString num  _ =  
    let
	val _ = print ("\nTest "^(Int.toString(num))^
		       " - ****** FATAL ERROR ******\n\n");
    in 
	FATAL
    end
    
fun foldl_map _ (x, []) = (x, [])
  | foldl_map f (x, y :: ys) =
    let
        val (x', y') = f (x, y);
        val (x'', ys') = foldl_map f (x', ys);
    in (x'', y' :: ys') end;
    
fun filter (pred: 'a->bool) : 'a list -> 'a list =
    let fun filt [] = []
          | filt (x :: xs) = if pred x then x :: filt xs else filt xs
    in filt end;

fun printList resultToString results  = 
    let	
	val _ = print("\n\n")
	val _ = print( "Test Results:\n") 
	val _ = print( "=============\n") 
	val (num,testres) = (foldl_map (fn (num,res) => 
					   (num+1,(printResult 
						       resultToString 
						       num res) )) 
				       (0, results));
	val _ = print ("\n\n");
	val _ = print ("Summary:\n");
	val _ = print ("--------\n");
	val _ = print ("Number successful tests cases: ")
	val suc = length (filter (fn (r) => (r=SUCCESS)) testres)
	val _ = print (Int.toString(suc)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((suc * 100) div num)^"%)\n")
	val _ = print ("Number of warnings:             ")
	val warn = length (filter (fn (r) => (r=WARNING)) testres)
	val _ = print (Int.toString(warn)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((warn * 100) div num)^"%)\n")
	val _ = print ("Number of errors:               ")
	val err = length (filter (fn (r) => (r=ERROR)) testres)
	val _ = print (Int.toString(err)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((err * 100) div num)^"%)\n")
	val _ = print ("Number of failures:             ")
	val fail = length (filter (fn (r) => (r=FAILURE)) testres)
	val _ = print (Int.toString(fail)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((fail * 100) div num)^"%)\n")
	val _ = print ("Number of fatal errors:         ")
	val fatal = length (filter (fn (r) => (r=FATAL)) testres)
	val _ = print (Int.toString(fatal)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((fatal * 100) div num)^"%)\n")
	val _ = print ("\nOverall result: ")
	val _ = if(suc=num) 
		then print ("success")
		else print ("failed ")
	val _ = print ("\n===============\n\n")
    in
	()
    end

fun string_to_char "\255" = chr 255
  | string_to_char "\254" = chr 254
  | string_to_char "\253" = chr 253
  | string_to_char "\252" = chr 252
  | string_to_char "\251" = chr 251
  | string_to_char "\250" = chr 250
  | string_to_char "\249" = chr 249
  | string_to_char "\248" = chr 248
  | string_to_char "\247" = chr 247
  | string_to_char "\246" = chr 246
  | string_to_char "\245" = chr 245
  | string_to_char "\244" = chr 244
  | string_to_char "\243" = chr 243
  | string_to_char "\242" = chr 242
  | string_to_char "\241" = chr 241
  | string_to_char "\240" = chr 240
  | string_to_char "\239" = chr 239
  | string_to_char "\238" = chr 238
  | string_to_char "\237" = chr 237
  | string_to_char "\236" = chr 236
  | string_to_char "\235" = chr 235
  | string_to_char "\234" = chr 234
  | string_to_char "\233" = chr 233
  | string_to_char "\232" = chr 232
  | string_to_char "\231" = chr 231
  | string_to_char "\230" = chr 230
  | string_to_char "\229" = chr 229
  | string_to_char "\228" = chr 228
  | string_to_char "\227" = chr 227
  | string_to_char "\226" = chr 226
  | string_to_char "\225" = chr 225
  | string_to_char "\224" = chr 224
  | string_to_char "\223" = chr 223
  | string_to_char "\222" = chr 222
  | string_to_char "\221" = chr 221
  | string_to_char "\220" = chr 220
  | string_to_char "\219" = chr 219
  | string_to_char "\218" = chr 218
  | string_to_char "\217" = chr 217
  | string_to_char "\216" = chr 216
  | string_to_char "\215" = chr 215
  | string_to_char "\214" = chr 214
  | string_to_char "\213" = chr 213
  | string_to_char "\212" = chr 212
  | string_to_char "\211" = chr 211
  | string_to_char "\210" = chr 210
  | string_to_char "\209" = chr 209
  | string_to_char "\208" = chr 208
  | string_to_char "\207" = chr 207
  | string_to_char "\206" = chr 206
  | string_to_char "\205" = chr 205
  | string_to_char "\204" = chr 204
  | string_to_char "\203" = chr 203
  | string_to_char "\202" = chr 202
  | string_to_char "\201" = chr 201
  | string_to_char "\200" = chr 200
  | string_to_char "\199" = chr 199
  | string_to_char "\198" = chr 198
  | string_to_char "\197" = chr 197
  | string_to_char "\196" = chr 196
  | string_to_char "\195" = chr 195
  | string_to_char "\194" = chr 194
  | string_to_char "\193" = chr 193
  | string_to_char "\192" = chr 192
  | string_to_char "\191" = chr 191
  | string_to_char "\190" = chr 190
  | string_to_char "\189" = chr 189
  | string_to_char "\188" = chr 188
  | string_to_char "\187" = chr 187
  | string_to_char "\186" = chr 186
  | string_to_char "\185" = chr 185
  | string_to_char "\184" = chr 184
  | string_to_char "\183" = chr 183
  | string_to_char "\182" = chr 182
  | string_to_char "\181" = chr 181
  | string_to_char "\180" = chr 180
  | string_to_char "\179" = chr 179
  | string_to_char "\178" = chr 178
  | string_to_char "\177" = chr 177
  | string_to_char "\176" = chr 176
  | string_to_char "\175" = chr 175
  | string_to_char "\174" = chr 174
  | string_to_char "\173" = chr 173
  | string_to_char "\172" = chr 172
  | string_to_char "\171" = chr 171
  | string_to_char "\170" = chr 170
  | string_to_char "\169" = chr 169
  | string_to_char "\168" = chr 168
  | string_to_char "\167" = chr 167
  | string_to_char "\166" = chr 166
  | string_to_char "\165" = chr 165
  | string_to_char "\164" = chr 164
  | string_to_char "\163" = chr 163
  | string_to_char "\162" = chr 162
  | string_to_char "\161" = chr 161
  | string_to_char "\160" = chr 160
  | string_to_char "\159" = chr 159
  | string_to_char "\158" = chr 158
  | string_to_char "\157" = chr 157
  | string_to_char "\156" = chr 156
  | string_to_char "\155" = chr 155
  | string_to_char "\154" = chr 154
  | string_to_char "\153" = chr 153
  | string_to_char "\152" = chr 152
  | string_to_char "\151" = chr 151
  | string_to_char "\150" = chr 150
  | string_to_char "\149" = chr 149
  | string_to_char "\148" = chr 148
  | string_to_char "\147" = chr 147
  | string_to_char "\146" = chr 146
  | string_to_char "\145" = chr 145
  | string_to_char "\144" = chr 144
  | string_to_char "\143" = chr 143
  | string_to_char "\142" = chr 142
  | string_to_char "\141" = chr 141
  | string_to_char "\140" = chr 140
  | string_to_char "\139" = chr 139
  | string_to_char "\138" = chr 138
  | string_to_char "\137" = chr 137
  | string_to_char "\136" = chr 136
  | string_to_char "\135" = chr 135
  | string_to_char "\134" = chr 134
  | string_to_char "\133" = chr 133
  | string_to_char "\132" = chr 132
  | string_to_char "\131" = chr 131
  | string_to_char "\130" = chr 130
  | string_to_char "\129" = chr 129
  | string_to_char "\128" = chr 128
  | string_to_char "\127" = chr 127
  (* *)
  | string_to_char "\\" = chr 92
  (* *)  
  | string_to_char "\^_" = chr 31
  | string_to_char "\^^" = chr 30
  | string_to_char "\^]" = chr 29
  | string_to_char "\^\" = chr 28 (* " *)
  | string_to_char "\^[" = chr 27
  | string_to_char "\^Z" = chr 26
  | string_to_char "\^Y" = chr 25
  | string_to_char "\^X" = chr 24
  | string_to_char "\^W" = chr 23
  | string_to_char "\^V" = chr 22
  | string_to_char "\^U" = chr 21
  | string_to_char "\^T" = chr 20
  | string_to_char "\^S" = chr 19
  | string_to_char "\^R" = chr 18
  | string_to_char "\^Q" = chr 17
  | string_to_char "\^P" = chr 16
  | string_to_char "\^O" = chr 15
  | string_to_char "\^N" = chr 14
  | string_to_char "\^M" = chr 13
  | string_to_char "\^L" = chr 12
  | string_to_char "\^K" = chr 11
  | string_to_char "\^J" = chr 10
  | string_to_char "\^I" = chr 9
  | string_to_char "\^H" = chr 8
  | string_to_char "\^G" = chr 7
  | string_to_char "\^F" = chr 6
  | string_to_char "\^E" = chr 5
  | string_to_char "\^D" = chr 4
  | string_to_char "\^C" = chr 3
  | string_to_char "\^B" = chr 2
  | string_to_char "\^A" = chr 1
  | string_to_char "\^@" = chr 0
  | string_to_char s   = (Option.valOf o Char.fromCString) s 
  

end 	




 
