(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * harness.sml --- the HOL-TestGen test harness
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2007 ETH Zurich, Switzerland
 *               2011 Achim D. Brucker
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: harnessng.fs 9200 2011-07-14 05:53:07Z brucker $ *)

namespace HolTestGen
    module TestHarness =
type testresult = SUCCESS
                    | WARNING
                    | FAILURE
                    | ERROR
                    | FATAL

let rsf = false


let init () = System.Console.Write "NOT YET IMPLEMENTED"
             
   let check retlet prelist testpred =
         let fst (x,y) = x
         let pre = try
                     (Some(List.fold (fun x y -> x && y) true (List.map (fun x -> (x () = true)) prelist )), None)
                   with
                     | exn -> (None,Some(exn))
         let post = try
                      (Some((testpred()=true)),retlet(),None)
                    with
                      | exn -> (None,None,Some(exn))
         (pre,post)



let printResult num ((SOME(true),NONE),(SOME(true),NONE)) = System.Console.Write ("Test "^(Int.toString(num))^" -     SUCCESS\n")
                SUCCESS

  | printResult num ((SOME(true),NONE),(SOME(false),NONE)) =  
    let
       let _ = System.Console.Write ("Test "^(Int.toString(num))
                     ^" - *** FAILURE: post-condition false\n")
    in 
       FAILURE
    end
  | printResult  num ((SOME(true),NONE),(NONE,SOME(e_post))) = 
    let
       let _ = System.Console.Write ("Test "^(Int.toString(num))
                     ^" - *** FAILURE: exception during post-condition\n")
    in 
       FAILURE
    end
  | printResult num ((SOME(false),NONE),(SOME(true),NONE))=  
    let
       let _ = System.Console.Write ("Test "^(Int.toString(num))
                     ^" -     WARNING: pre-condition false "^
                     "(post-condition true)\n")
    in 
       WARNING
    end
  | printResult num ((SOME(false),NONE),(SOME(false),NONE)) =
    let
       let _ = System.Console.Write ("Test "^(Int.toString(num))
                     ^" - *   WARNING: pre-condition false "^
                     "(post-condition false)\n")
    in 
       WARNING
    end
  | printResult num ((SOME(false),NONE),(NONE,SOME(e_post))) =
    let
       let _ = System.Console.Write ("Test "^(Int.toString(num))^
                     " - **  WARNING: pre-condition false "^
                     "(exception during post_condition)\n")
    in 
       WARNING
    end
  | printResult num ((NONE,SOME(e_pre)),(SOME(true),NONE)) =  
    let
       let _ = System.Console.Write ("Test "^(Int.toString(num))^
                     " - **  ERROR:exception during pre-condition "^
                     "(post-condition true)\n")
    in 
       ERROR
    end
  | printResult num ((NONE,SOME(e_pre)),(SOME(false),NONE)) = 
    let
       let _ = System.Console.Write ("Test "^(Int.toString(num))^
                     " - **  ERROR: exception during pre-condition "^
          "(post-condition false)\n")
    in 
       ERROR
    end
  | printResult num ((NONE,SOME(e_pre)),(NONE,SOME(e_post))) =
    let
       let _ = System.Console.Write ("Test "^(Int.toString(num))^
                     " - **  ERROR: exception during pre-condition "^
                     "and post-condition\n")
    in 
       ERROR
    end
  | printResult num  _ =  
    let
       let _ = System.Console.Write ("\nTest "^(Int.toString(num))^
                     " - ****** FATAL ERROR ******\n\n")
    in 
       FATAL
    end
    
let foldl_map _ (x, []) = (x, [])
  | foldl_map f (x, y :: ys) =
    let
        let (x', y') = f (x, y);
        let (x'', ys') = foldl_map f (x', ys);
    in (x'', y' :: ys') end;
    
let filter (pred: 'a->bool) : 'a list -> 'a list =
    let let filt [] = []
          | filt (x :: xs) = if pred x then x :: filt xs else filt xs
    in filt end;

let printList results  = 
    let       
       let _ = System.Console.Write("\n\n")
       let _ = System.Console.Write( "Test Results:\n") 
       let _ = System.Console.Write( "=============\n") 
       let (num,testres) = (foldl_map (fn (num,res) => 
                                      (num+1,(printResult  
                                                 num res) )) 
                                   (0, results));
       let _ = System.Console.Write ("\n\n");
       let _ = System.Console.Write ("Summary:\n");
       let _ = System.Console.Write ("--------\n");
       let _ = System.Console.Write ("Number successful tests cases: ")
       let suc = length (filter (fn (r) => (r=SUCCESS)) testres)
       let _ = System.Console.Write (Int.toString(suc)^" of "^Int.toString(num)^" ")
       let _ = System.Console.Write ("(ca. "^Int.toString((suc * 100) div num)^"%)\n")
       let _ = System.Console.Write ("Number of warnings:             ")
       let warn = length (filter (fn (r) => (r=WARNING)) testres)
       let _ = System.Console.Write (Int.toString(warn)^" of "^Int.toString(num)^" ")
       let _ = System.Console.Write ("(ca. "^Int.toString((warn * 100) div num)^"%)\n")
       let _ = System.Console.Write ("Number of errors:               ")
       let err = length (filter (fn (r) => (r=ERROR)) testres)
       let _ = System.Console.Write (Int.toString(err)^" of "^Int.toString(num)^" ")
       let _ = System.Console.Write ("(ca. "^Int.toString((err * 100) div num)^"%)\n")
       let _ = System.Console.Write ("Number of failures:             ")
       let fail = length (filter (fn (r) => (r=FAILURE)) testres)
       let _ = System.Console.Write (Int.toString(fail)^" of "^Int.toString(num)^" ")
       let _ = System.Console.Write ("(ca. "^Int.toString((fail * 100) div num)^"%)\n")
       let _ = System.Console.Write ("Number of fatal errors:         ")
       let fatal = length (filter (fn (r) => (r=FATAL)) testres)
       let _ = System.Console.Write (Int.toString(fatal)^" of "^Int.toString(num)^" ")
       let _ = System.Console.Write ("(ca. "^Int.toString((fatal * 100) div num)^"%)\n")
       let _ = System.Console.Write ("\nOverall result: ")
       let _ = if(suc=num) 
              then System.Console.Write ("success")
              else System.Console.Write ("failed ")
       let _ = System.Console.Write ("\n===============\n\n")
    in
       ()
    end


let run test_cases = printList (checkList test_cases)


end        



 
